@@ -1158,7 +1158,7 @@ void GR_Win32USPGraphics::prepareToRenderChars(GR_RenderInfo & ri)
 	UT_uint32 iPoints = (UT_uint32)pFont->getPointSize();
 #endif
 	
-	if(iZoom == RI.m_iZoom && RI.s_pOwnerDraw == & ri && pFont->getPrintDC() != getPrintDC())
+	if(iZoom == RI.m_iZoom && RI.s_pOwnerDraw == & ri && pFont->getPrintDC() == getPrintDC())
 	{
 		// the buffer is up-to-date
 		return;
@@ -1455,7 +1455,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	// store the print DC used to measure the font in the font ...
 	pFont->setPrintDC(getPrintDC());
 	
-	HDC hdc = 0;
+	HDC hdc = 0; // this is the hdc to use if all fails
+	HDC hdc1 = 0; // this is the hdc to use in the first pass (null if we have script cache)
 	if(*(pFont->getScriptCache()) == NULL)
 	{
 		// need to make sure that the HDC has the correct font set
@@ -1470,10 +1471,12 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 
 		// we remember the hdc for which we measured so we can remeasure when hdc changes
 		RI.m_hdc = hdc;
+		hdc1 = hdc;
 	}
 	else
 	{
 		hdc = pFont->getPrintDC() ? pFont->getPrintDC() : m_hdc;
+		hdc1 = 0;
 		RI.m_hdc = hdc;
 	}
 	
@@ -1484,13 +1487,12 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	if(RI.m_bShapingFailed)
 		pItem->m_si.a.eScript = GRScriptType_Undefined;
 	
-	HRESULT hRes = fScriptPlace(hdc, pFont->getScriptCache(), RI.m_pIndices,
+	HRESULT hRes = fScriptPlace(hdc1, pFont->getScriptCache(), RI.m_pIndices,
 								RI.m_iIndicesCount, RI.m_pVisAttr,
 								& pItem->m_si.a, RI.m_pAdvances, RI.m_pGoffsets, & RI.m_ABC);
 
 	if( hRes == E_PENDING)
 	{
-		UT_ASSERT_HARMLESS( hdc == 0 );
 		_setupFontOnDC(pFont, false);
 		bFontSetUpOnDC = true;
 		
@@ -1502,9 +1504,61 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 		hRes = fScriptPlace(hdc, pFont->getScriptCache(), RI.m_pIndices,
 							RI.m_iIndicesCount, RI.m_pVisAttr,
 							& pItem->m_si.a, RI.m_pAdvances, RI.m_pGoffsets, & RI.m_ABC);
+
+		UT_ASSERT_HARMLESS( !hRes );
+	}
+	else if(hRes)
+	{
+		// some other error, at least set the advances and offsets to 0
+		memset(RI.m_pAdvances, 0, sizeof(int) * RI.m_iIndicesCount);
+		memset(RI.m_pGoffsets, 0, sizeof(GOFFSET) * RI.m_iIndicesCount);
+		memset(&RI.m_ABC, 0, sizeof(ABC));
 	}
 	
 
+	const UT_uint32 iAdvSize = 80;
+	int iAdvances[iAdvSize];
+	GOFFSET stGoffsets[iAdvSize];
+	UT_uint32 iCount = iAdvSize;
+	ABC stABC;
+
+	int * pAdvances = &iAdvances[0];
+	GOFFSET * pGoffsets = &stGoffsets[0];
+
+	bool bAdjustAdvances = false;
+	
+#if 1
+	if(!m_bPrint && getPrintDC())
+	{
+		// This is a trick to avoid unseemly gaps between letters on screen due to
+		// discrepancies between font metrics of screen and printer. We measure the font
+		// also on screen and distribute the difference to the left and right of each
+		// character by adjusting each character's m_pGoffsets.du. This improves things,
+		// but does not completely remove the problem; whether it is worth the extra
+		// is to be seen. Tomas, Apr 9, 2005
+		_setupFontOnDC(pFont, true);
+		
+		if(iCount <= RI.m_iIndicesCount)
+		{
+			iCount = RI.m_iIndicesCount + 1;
+			pAdvances = new int[iCount];
+			pGoffsets = new GOFFSET[iCount];
+		}
+		
+		
+		SCRIPT_CACHE sc = NULL;
+		HRESULT hRes = fScriptPlace(m_hdc, &sc, RI.m_pIndices, RI.m_iIndicesCount, RI.m_pVisAttr,
+									& pItem->m_si.a, pAdvances, pGoffsets, & stABC);
+
+		UT_ASSERT_HARMLESS( !hRes );
+		if(!hRes)
+			bAdjustAdvances = true;
+
+		if(sc)
+			fScriptFreeCache(&sc);			
+	}
+#endif
+	
 	pItem->m_si.a.eScript = eScript;
 
 	// remember the zoom at which we calculated this ...
@@ -1526,6 +1580,7 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	double dWidth = 0;
 	double dDeviceWidth = 0;
 	double dPrevDeviceWidth = 0;
+	double dAdjustment = 0;
 	
 	if(getPrintDC())
 	{
@@ -1533,22 +1588,25 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
-			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
 			dWidth += (double)RI.m_pAdvances[i];
-			dDeviceWidth = dWidth*(double)getResolution()
-									   /((double)m_nPrintLogPixelsY*fXYRatio);
-			RI.m_pAdvances[i] = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
-			dPrevDeviceWidth = dDeviceWidth;
+			dDeviceWidth = dWidth*(double)getResolution() / ((double)m_nPrintLogPixelsY*fXYRatio);
 
+			if(bAdjustAdvances)
+			{
+				dAdjustment = (((double)RI.m_pAdvances[i]*(double)getResolution()
+								/((double)m_nPrintLogPixelsY*fXYRatio))
+							   - (double)pAdvances[i]*(double)getResolution()*100.
+							   /((double)getDeviceResolution()*(double)iZoom * m_fXYRatio))/2.;
+			}
+			RI.m_pAdvances[i]   = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
 			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()
-											/((double)m_nPrintLogPixelsY*fXYRatio) + 0.5);
+											/((double)m_nPrintLogPixelsY*fXYRatio) + 2*dAdjustment + 0.5);
 			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()
 											/((double)m_nPrintLogPixelsY) + 0.5);
+
+			dPrevDeviceWidth = dDeviceWidth;
 		}
 
-		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
-		//RI.m_ABC.abcB = tlu(RI.m_ABC.abcB);
-		//RI.m_ABC.abcC = tlu(RI.m_ABC.abcC);
 		RI.m_ABC.abcA = (UT_sint32)((double) RI.m_ABC.abcA * (double)getResolution()
 									/((double)m_nPrintLogPixelsY*fXYRatio) + 0.5);
 		RI.m_ABC.abcB = (UT_sint32)((double) RI.m_ABC.abcB * (double)getResolution()
@@ -1560,7 +1618,6 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
-			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
 			dWidth += (double)RI.m_pAdvances[i];
 			dDeviceWidth = dWidth*(double)getResolution()/
 									   ((double)getDeviceResolution()*m_fXYRatio);
@@ -1574,9 +1631,6 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 											((double)getDeviceResolution()) + 0.5);
 		}
 
-		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
-		//RI.m_ABC.abcB = tlu(RI.m_ABC.abcB);
-		//RI.m_ABC.abcC = tlu(RI.m_ABC.abcC);
 		RI.m_ABC.abcA = (UT_sint32)((double) RI.m_ABC.abcA * (double)getResolution() /
 									((double)getDeviceResolution()*m_fXYRatio) + 0.5);
 		RI.m_ABC.abcB = (UT_sint32)((double) RI.m_ABC.abcB * (double)getResolution() /
@@ -1588,23 +1642,22 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
-			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
 			dWidth += (double)RI.m_pAdvances[i];
 			dDeviceWidth = dWidth*(double)getResolution()/
 									   ((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio);
-			
-			RI.m_pAdvances[i] = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
-			dPrevDeviceWidth = dDeviceWidth;
+
+			RI.m_pAdvances[i]   = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
 
 			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()/
-											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio) + 0.5);
+											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio)
+											   + 0.5);
+			
 			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()/
 											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING) + 0.5);
+			dPrevDeviceWidth = dDeviceWidth;
+
 		}
 
-		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
-		//RI.m_ABC.abcB = tlu(RI.m_ABC.abcB);
-		//RI.m_ABC.abcC = tlu(RI.m_ABC.abcC);
 		RI.m_ABC.abcA = (UT_sint32)((double) RI.m_ABC.abcA * (double)getResolution() /
 									((double)getDeviceResolution() * (double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio) + 0.5);
 		RI.m_ABC.abcB = (UT_sint32)((double) RI.m_ABC.abcB * (double)getResolution() /
@@ -1614,8 +1667,13 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	}
 	
 	RI.m_bRejustify = true;
-	
 
+	if(iCount != iAdvSize)
+	{
+		delete[] pAdvances;
+		delete[] pGoffsets;
+	}
+	
 	if(hRes)
 	{
 		UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
@@ -1996,16 +2054,7 @@ UT_sint32 GR_Win32USPGraphics::countJustificationPoints(const GR_RenderInfo & ri
 	// now we need to make sense of the stats
 	if(iCountKashida)
 	{
-		RI.m_eJustification = (SCRIPT_JUSTIFY)(
-			                  SCRIPT_JUSTIFY_ARABIC_NORMAL
-                			& SCRIPT_JUSTIFY_ARABIC_KASHIDA
-			                & SCRIPT_JUSTIFY_ARABIC_ALEF
-			                & SCRIPT_JUSTIFY_ARABIC_HA
- 			                & SCRIPT_JUSTIFY_ARABIC_RA
-			                & SCRIPT_JUSTIFY_ARABIC_BA
-                			& SCRIPT_JUSTIFY_ARABIC_BARA
-			                & SCRIPT_JUSTIFY_ARABIC_SEEN
-			                & SCRIPT_JUSTIFY_BLANK );
+		RI.m_eJustification = SCRIPT_JUSTIFY_ARABIC_KASHIDA;
 		
 		return iCountSpace + iCountKashida;
 	}
@@ -2067,7 +2116,9 @@ void GR_Win32USPGraphics::justify(GR_RenderInfo & ri)
 	for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 	{
 		UT_uint32 k = pItem->m_si.a.fRTL ? RI.m_iIndicesCount - i - 1: i;
-		if(RI.m_pVisAttr[k].uJustification & RI.m_eJustification)
+		if(RI.m_pVisAttr[k].uJustification == RI.m_eJustification ||
+		   (RI.m_eJustification == SCRIPT_JUSTIFY_ARABIC_KASHIDA &&
+			RI.m_pVisAttr[k].uJustification >= SCRIPT_JUSTIFY_ARABIC_KASHIDA))
 		{
 			UT_uint32 iSpace = iExtraSpace/iPoints;
 			iExtraSpace -= iSpace;
@@ -2237,6 +2288,37 @@ void GR_Win32USPGraphics::drawChars(const UT_UCSChar* pChars,
 		delete [] pwChars;
 }
 
+void GR_Win32USPGraphics::setZoomPercentage(UT_uint32 iZoom)
+{
+	if(getZoomPercentage() != iZoom)
+	{
+		
+		GR_Graphics::setZoomPercentage(iZoom);
+
+		// now force remeasuring of character widths in any views that are associated with
+		// this graphics
+		XAP_App * pApp = XAP_App::getApp();
+		UT_return_if_fail( pApp );
+
+		UT_uint32 iFrameCount = pApp->getFrameCount();
+		for(UT_uint32 i = 0; i < iFrameCount; i++)
+		{
+			XAP_Frame * pFrame = pApp->getFrame(i);
+			if(!pFrame)
+				continue;
+
+			AV_View * pView = pFrame->getCurrentView();
+			if(pView)
+			{
+				GR_Graphics * pG = pView->getGraphics();
+
+				if(pG == this)
+					pView->remeasureCharsWithoutRebuild();
+			}
+		}
+	}
+}
+
 
 //////////////////////////////////////////////////////////////////////////////
 //
@@ -2442,7 +2524,7 @@ GR_Win32USPFont::GR_Win32USPFont(LOGFONT & lf, double fPoints, HDC hdc, HDC prin
 	  m_printHDC(NULL),
 	  m_iScreenAscent(0)
 {
-};
+}
 
 void GR_Win32USPFont::_clearAnyCachedInfo()
 {
