@@ -36,6 +36,8 @@ dcl(MSG_HistoryPartRestore4, "To quit the restoration attempt, press Cancel.")
 dcl(MSG_HistoryNoRestore, "AbiWord cannot restore version %d of the document because the version information is missing.")
 dcl(MSG_HistoryConfirmSave, "You have to save changes to document %s before proceeding. Save now?")
 
+dcl(MSG_NoUndo, "This operation cannot be undone. Are you sure you want to proceed?")
+
 /* Default name for new, untitled document */
 dcl(UntitledDocument,			"Untitled%d")
 
@@ -132,7 +134,7 @@ dcl(DLG_UFS_UnderlineCheck, 	"Underline")
 dcl(DLG_UFS_OverlineCheck,		"Overline")
 dcl(DLG_UFS_HiddenCheck,        "Hidden")	
 dcl(DLG_UFS_TransparencyCheck,	"Set no Highlight Color")
-dcl(DLG_UFS_FontTab,			"	Font   ")
+dcl(DLG_UFS_FontTab,			"Font")
 dcl(DLG_UFS_ColorTab,			"Text Color")
 dcl(DLG_UFS_BGColorTab, 		"HighLight Color")
 dcl(DLG_UFS_StyleRegular,		"Regular")
@@ -153,8 +155,12 @@ dcl(DLG_FOSA_SaveAsTitle,				"Save File As")
 dcl(DLG_FOSA_ExportTitle, "Export File")
 dcl(DLG_FOSA_ImportTitle, "Import File")
 dcl(DLG_FOSA_InsertTitle, "Insert File")
+dcl(DLG_FOSA_InsertMath,  "Insert Math File")
+dcl(DLG_FOSA_InsertObject,  "Insert Embeddable Object")
 dcl(DLG_FOSA_PrintToFileTitle,			"Print To File")
 dcl(DLG_FOSA_FileOpenTypeLabel, 		"Open file as type:")
+dcl(DLG_FOSA_FileInsertMath, 		    "Insert MathML file:")
+dcl(DLG_FOSA_FileInsertObject, 		    "Insert Embeddable Object file:")
 dcl(DLG_FOSA_FileSaveTypeLabel, 		"Save file as type:")
 dcl(DLG_FOSA_FilePrintTypeLabel,		"Print file as type:")
 dcl(DLG_FOSA_FileTypeAutoDetect,		"Automatically Detected")
@@ -186,6 +192,9 @@ dcl(TB_Zoom_PageWidth,					"Page Width")
 dcl(TB_Zoom_WholePage,					"Whole Page")
 dcl(TB_Zoom_Percent, "Other...")
 
+/* Font tool bar*/
+dcl(TB_Font_Symbol,						"Symbols")
+
 /* Unix Print dialog */
 dcl(DLG_UP_PrintTitle,					"Print")
 dcl(DLG_UP_PrintPreviewTitle,					   "AbiWord: Print Preview")
@@ -268,12 +277,16 @@ dcl(DLG_Image_ImageDesc, "Set Image Name")
 dcl(DLG_Image_TextWrapping, "Define Text Wrapping")
 dcl(DLG_Image_Placement, "Define Image Placement")
 dcl(DLG_Image_InLine, "Image placed in-line (no text wrapping)")
+dcl(DLG_Image_WrappedNone, "Image floats above text")
 dcl(DLG_Image_WrappedRight, "Text wrapped to the Right of the Image")
 dcl(DLG_Image_WrappedLeft, "Text wrapped to the Left of the Image")
 dcl(DLG_Image_WrappedBoth, "Text wrapped on both sides of the Image")
 dcl(DLG_Image_PlaceParagraph, "Position relative to nearest paragraph")
 dcl(DLG_Image_PlaceColumn, "Position relative to its Column")
 dcl(DLG_Image_PlacePage, "Position relative to its Page")
+dcl(DLG_Image_WrapType, "Type of text wrapping")
+dcl(DLG_Image_SquareWrap, "Square text wrapping")
+dcl(DLG_Image_TightWrap, "Tight text wrapping")
 
 /* ListDocuments Dialog */
 dcl(DLG_LISTDOCS_Title,					"Opened Documents")
@@ -361,7 +374,6 @@ dcl(LANG_LA_IT, 							"Latin (Renaissance)")	// Is _IT the right thing here?
 dcl(LANG_LV_LV, 							"Latvian")
 dcl(LANG_LT_LT, 							"Lithuanian")
 dcl(LANG_MK,								"Macedonian")	// Jordi 19/10/2002 Hipi: Why not mk-MK?
-dcl(LANG_MS_MY,                             "Malay")
 dcl(LANG_MI_NZ,                             "Maori")
 dcl(LANG_MR_IN,                             "Marathi")
 dcl(LANG_MH_MH, 							"Marshallese (Marshall Islands)")
