@@ -97,7 +97,8 @@ fl_FrameLayout::fl_FrameLayout(FL_DocLayout* pLayout,
 	  m_iXPage(0),
 	  m_iYPage(0),
 	  m_iBoundingSpace(0),
-	  m_iFrameWrapMode(FL_FRAME_ABOVE_TEXT)
+	  m_iFrameWrapMode(FL_FRAME_ABOVE_TEXT),
+	  m_bIsTightWrap(false)
 {
 	UT_ASSERT(m_pDocSL->getContainerType() == FL_CONTAINER_DOCSECTION);
 }
@@ -153,6 +154,11 @@ void 	fl_FrameLayout::setContainerProperties(void)
 	pFrame->setRightStyle(m_lineRight );
 	pFrame->setXpad(m_iXpad);
 	pFrame->setYpad(m_iYpad);
+	pFrame->setTightWrapping(m_bIsTightWrap);
+	if(FL_FRAME_BELOW_TEXT ==  m_iFrameWrapMode)
+        {
+	        pFrame->setAbove(false);
+	}
 //
 // Now do the image for this frame.
 //
@@ -342,10 +348,26 @@ fl_SectionLayout * fl_FrameLayout::getSectionLayout(void) const
 bool fl_FrameLayout::doclistener_changeStrux(const PX_ChangeRecord_StruxChange * pcrxc)
 {
 	UT_ASSERT(pcrxc->getType()==PX_ChangeRecord::PXT_ChangeStrux);
+	fp_FrameContainer * pFrameC = static_cast<fp_FrameContainer *>(getFirstContainer());
+	UT_GenericVector<fl_BlockLayout *> vecBlocks;
+	pFrameC->getBlocksAroundFrame(vecBlocks);
+	UT_uint32 i = 0;
+	for(i=0; i< vecBlocks.getItemCount();i++)
+	{
+	  fl_BlockLayout * pBL = vecBlocks.getNthItem(i);
+	  pBL->collapse();
+	  xxx_UT_DEBUGMSG(("Collapse block %x \n",pBL));
+	}
 	setAttrPropIndex(pcrxc->getIndexAP());
 	collapse();
 	lookupProperties();
 	format();
+	for(i=0; i< vecBlocks.getItemCount();i++)
+	{
+	  fl_BlockLayout * pBL = vecBlocks.getNthItem(i);
+	  pBL->format();
+	  xxx_UT_DEBUGMSG(("Format block %x \n",pBL));
+	}
 	return true;
 }
 
@@ -374,12 +396,14 @@ void fl_FrameLayout::markAllRunsDirty(void)
 	}
 }
 
-void fl_FrameLayout::updateLayout(void)
+void fl_FrameLayout::updateLayout(bool bDoAll)
 {
+        xxx_UT_DEBUGMSG(("UpdsateLayout in in framelayout \n"));
 	if(needsReformat())
 	{
 		format();
 	}
+	m_vecFormatLayout.clear();
 	fl_ContainerLayout*	pBL = getFirstLayout();
 	while (pBL)
 	{
@@ -417,36 +441,29 @@ bool fl_FrameLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 		pFrameC->getPage()->markDirtyOverlappingRuns(pFrameC);
 	}
 #endif
+	fp_FrameContainer * pFrameC = static_cast<fp_FrameContainer *>(getFirstContainer());
+	UT_GenericVector<fl_BlockLayout *> vecBlocks;
+	pFrameC->getBlocksAroundFrame(vecBlocks);
+	UT_uint32 i = 0;
+	for(i=0; i< vecBlocks.getItemCount();i++)
+	{
+	  fl_BlockLayout * pBL = vecBlocks.getNthItem(i);
+	  pBL->collapse();
+	  xxx_UT_DEBUGMSG(("Collapse block %x \n",pBL));
+	}
+
 //
 // Remove all remaining structures
 //
 	collapse();
 //	UT_ASSERT(pcrx->getStruxType()== PTX_SectionFrame);
 //
-
-	fl_ContainerLayout * pPrev = getPrev();
-	fl_ContainerLayout * pNext = getNext();
-
-	if(pPrev != NULL)
-	{
-		pPrev->setNext(pNext);
-	}
-	else
-	{
-		myContainingLayout()->setFirstLayout(pNext);
-	}
-	if(pNext != NULL)
-	{
-		pNext->setPrev(pPrev);
-	}
-	else
-	{
-		myContainingLayout()->setLastLayout(pPrev);
-	}
+	fl_ContainerLayout * pCL = getPrev();
+	myContainingLayout()->remove(this);
+	UT_DEBUGMSG(("Unlinking frame Layout %x \n",this));
 //
 // Remove from the list of frames in the previous block
 //
-	fl_ContainerLayout * pCL = getPrev();
 	while(pCL && pCL->getContainerType() != FL_CONTAINER_BLOCK)
 	{
 		pCL = pCL->getPrev();
@@ -457,7 +474,42 @@ bool fl_FrameLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 		return false;
 	}
 	fl_BlockLayout * pBL = static_cast<fl_BlockLayout *>(pCL);
-	pBL->removeFrame(this);
+	bool bFound = false;
+	for(i=0; i<static_cast<UT_uint32>(pBL->getNumFrames()) && !bFound;i++)
+	{
+	  fl_FrameLayout * pF = pBL->getNthFrameLayout(i);
+	  if(pF == this)
+	  {
+	    bFound = true;
+	  }
+	}
+	if(bFound)
+	{
+	  pBL->removeFrame(this);
+	}
+	else
+	{
+	  UT_DEBUGMSG(("Whoops! not Frame found. Try ahead \n"));
+	  pCL = this;
+	  while(pCL && pCL->getContainerType() != FL_CONTAINER_BLOCK)
+	  {
+	    pCL = pCL->getNext();
+	  }
+	  if(pCL == NULL)
+	  {
+	    UT_DEBUGMSG(("No BlockLayout before this frame! \n"));
+	    return false;
+	  }
+	  pBL = static_cast<fl_BlockLayout *>(pCL);
+	  pBL->removeFrame(this);
+	}
+	for(i=0; i< vecBlocks.getItemCount();i++)
+	{
+	  pBL = vecBlocks.getNthItem(i);
+	  pBL->format();
+	  xxx_UT_DEBUGMSG(("format block %x \n",pBL));
+	}
+
 	delete this;			// TODO whoa!  this construct is VERY dangerous.
 
 	return true;
@@ -506,8 +558,10 @@ void fl_FrameLayout::_createFrameContainer(void)
 	// Now do Frame image
 
 	const PP_AttrProp* pSectionAP = NULL;
-	m_pLayout->getDocument()->getAttrProp(m_apIndex, &pSectionAP);
-
+	// This is a real NO-NO: must *always* call getAP()
+	// m_pLayout->getDocument()->getAttrProp(m_apIndex, &pSectionAP);
+	getAP(pSectionAP);
+	
 	const XML_Char * pszDataID = NULL;
 	pSectionAP->getAttribute(PT_STRUX_IMAGE_DATAID, (const XML_Char *&)pszDataID);
 	DELETEP(m_pGraphicImage);
@@ -523,7 +577,7 @@ void fl_FrameLayout::_createFrameContainer(void)
 
 /*!
   Create a new Frame container.
-  \params If pPrevFrame is non-null place the new cell after this in the linked
+  \param If pPrevFrame is non-null place the new cell after this in the linked
           list, otherwise just append it to the end.
   \return The newly created Frame container
 */
@@ -562,12 +616,18 @@ void fl_FrameLayout::miniFormat(void)
 void fl_FrameLayout::format(void)
 {
 	xxx_UT_DEBUGMSG(("SEVIOR: Formatting first container is %x \n",getFirstContainer()));
+	if(isHidden() > FP_VISIBLE)
+	{
+		xxx_UT_DEBUGMSG(("Don't format FRAME coz I'm hidden! \n"));
+		return;
+	}
+
 	if(getFirstContainer() == NULL)
 	{
 		getNewContainer();
 	}
 	fl_ContainerLayout*	pBL = getFirstLayout();
-	
+	bool bOnPage = false;
 	while (pBL)
 	{
 		pBL->format();
@@ -638,10 +698,25 @@ void fl_FrameLayout::format(void)
 			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 			return;
 		}
-		pBL->setFramesOnPage(NULL);
+		if(!pBL->isCollapsed())
+		{
+		  bOnPage = pBL->setFramesOnPage(NULL);
+		  if(!bOnPage)
+		  {
+		    setNeedsReformat(this);
+		  }
+		}
+	}
+	m_bNeedsFormat = bOnPage;
+	m_bNeedsReformat = bOnPage;
+	if(!bOnPage)
+	{
+	  setNeedsReformat(this);
+	}
+	if(!bOnPage)
+	{
+	  return;
 	}
-	m_bNeedsFormat = false;
-	m_bNeedsReformat = false;
 	fl_DocSectionLayout * pDSL = getDocSectionLayout();
 	fp_FrameContainer * pFC = static_cast<fp_FrameContainer *>(getFirstContainer());
 	if(pFC)
@@ -650,6 +725,11 @@ void fl_FrameLayout::format(void)
 	}
 }
 
+void fl_FrameLayout::setNeedsReformat(fl_ContainerLayout * pCL,UT_uint32 offset)
+{
+        m_bNeedsReformat = true;
+        myContainingLayout()->setNeedsReformat(this);
+}
 /*!
     this function is only to be called by fl_ContainerLayout::lookupProperties()
     all other code must call lookupProperties() instead
@@ -678,6 +758,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	const XML_Char * pszBorderWidth = NULL;
 
 	const XML_Char * pszBoundingSpace = NULL;
+	const XML_Char * pszTightWrapped = NULL;
 // Frame Type
 
 	if(!pSectionAP || !pSectionAP->getProperty("frame-type",pszFrameType))
@@ -757,6 +838,21 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		m_iFrameWrapMode = FL_FRAME_ABOVE_TEXT;
 	}
+	//
+	// Wrap type
+	//
+	if(!pSectionAP || !pSectionAP->getProperty("tight-wrap",pszTightWrapped))
+	{
+		m_bIsTightWrap = false;
+	}
+	else if(strcmp(pszTightWrapped,"1") == 0)
+	{
+		m_bIsTightWrap = true;
+	}
+	else
+	{
+		m_bIsTightWrap = false;
+	}
 
 // Xpos
 
