@@ -96,7 +96,7 @@ fg_FillType * fp_Page::getFillType(void)
 
 bool fp_Page::isEmpty(void) const
 {
-	if((m_vecColumnLeaders.getItemCount() == 0) && (m_vecFootnotes.getItemCount() == 0) && (m_vecFrames.getItemCount() == 0))
+	if((m_vecColumnLeaders.getItemCount() == 0) && (m_vecFootnotes.getItemCount() == 0) && (m_vecAboveFrames.getItemCount() == 0) && (m_vecBelowFrames.getItemCount() == 0))
 	{
 		return true;
 	}
@@ -163,6 +163,10 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 	UT_sint32 iPage = getDocLayout()->findPage(this);
 	UT_DEBUGMSG(("Wrap passes = %d page %x page number %d \n",m_iCountWrapPasses,this,iPage ));
 #endif
+	if(m_iCountWrapPasses > 100)
+        {
+	  UT_DEBUGMSG(("Number of wrapped passed = %d \n",m_iCountWrapPasses));
+	}
 	UT_sint32 i= 0;
 	UT_sint32 nWrapped = 0;
 	fp_Container * pFirst = NULL;
@@ -185,9 +189,9 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 		}
 	}
 	UT_sint32 nWrappedObjs = 0;
-	for(i=0; i< static_cast<UT_sint32>(countFrameContainers()); i++)
+	for(i=0; i< static_cast<UT_sint32>(countAboveFrameContainers()); i++)
 	{
-		fp_FrameContainer * pFrame = getNthFrameContainer(i);
+		fp_FrameContainer * pFrame = getNthAboveFrameContainer(i);
 		if(pFrame->isWrappingSet())
 		{
 			nWrappedObjs++;
@@ -217,6 +221,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 					UT_Rect recLeft;
 					UT_Rect recRight;
 					pLine->genOverlapRects(recLeft,recRight);
+					bool bFoundOne = false;
 					if(recLeft.width == 0 && recRight.width == 0)
 					{
 						pLine->setWrapped(false);
@@ -283,13 +288,51 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 					else
 					{
 						pLine->setWrapped(true);
-						if(pLine->getPrev() && !pLine->isSameYAsPrevious())
+						fp_Line * pPrev = static_cast<fp_Line *>(pLine->getPrev());
+ 						if(pPrev && !pLine->isSameYAsPrevious())
+ 						{
+ 							if(pPrev->getY() == pLine->getY())
+ 							{
+ 								pLine->setSameYAsPrevious(true);
+ 							}
+ 						}
+						if(pLine->isSameYAsPrevious())
 						{
-							fp_Line * pPrev = static_cast<fp_Line *>(pLine->getPrev());
-							if(pPrev->getY() == pLine->getY())
-							{
-								pLine->setSameYAsPrevious(true);
-							}
+						  //
+						  // Look for a gap between lines.
+						  //
+						  UT_Rect recBetween;
+						  UT_Rect * pPrevRec = pPrev->getScreenRect();
+						  UT_Rect * pCurRec = pLine->getScreenRect();
+						  recBetween.left = pPrevRec->left+pPrevRec->width;
+						  recBetween.width = pCurRec->left - recBetween.left;
+						  if(pPrevRec->height != pCurRec->height)
+						  {
+						         pLine = pPrev;
+							 j--;
+							 bFoundOne = true;
+						  }
+						  else
+						  {
+						         recBetween.height = pPrevRec->height;
+							 recBetween.top = pPrevRec->top;
+							 if(!overlapsWrappedFrame(recBetween))
+							 {
+							      UT_DEBUGMSG(("Found a gap! \n"));
+							      pLine = pPrev;
+							      j--;
+							      bFoundOne = true;
+							 }
+							 else if(recBetween.width < 60)
+							 {
+							      UT_DEBUGMSG(("Found a tiny gap! %d \n",recBetween.width));
+							      pLine = pPrev;
+							      j--;
+							      bFoundOne = true;
+							 }
+						  }
+						  delete pPrevRec;
+						  delete pCurRec;
 						}
 					}
 					if(bFormatAllWrapped)
@@ -339,7 +382,6 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 // OK look to see if this line either overlaps a wrapped object or has
 // some space where a wrapped object should be
 //
-						bool bFoundOne = false;
 						fp_Line * pPrev = static_cast<fp_Line *>(pLine->getPrev());
 						if(pLine->isWrapped())
 						{
@@ -370,7 +412,43 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 								{
 									xxx_UT_DEBUGMSG(("Found wrapped line with extra  space %x  \n",pLine));
 									bFoundOne = true;
+									if(pPrev && pLine->isSameYAsPrevious())
+									{
+									  pLine = pPrev;
+									  j--;
+									}
 								}
+								else if (pPrev &&  pLine->isSameYAsPrevious())
+								{
+								  //
+								  // Look for a gap between lines.
+								  //
+								  UT_Rect recBetween;
+								  UT_Rect * pPrevRec = pPrev->getScreenRect();
+								  UT_Rect * pCurRec = pLine->getScreenRect();
+								  recBetween.left = pPrevRec->left+pPrevRec->width;
+								  recBetween.width = pCurRec->left - recBetween.left;
+								  if(pPrevRec->height != pCurRec->height)
+								  {
+								    pLine = pPrev;
+								    j--;
+								    bFoundOne = true;
+								  }
+								  else
+								  {
+								    recBetween.height = pPrevRec->height;
+								    recBetween.top = pPrevRec->top;
+								    if(!overlapsWrappedFrame(recBetween))
+								    {
+								      pLine = pPrev;
+								      j--;
+								      bFoundOne = true;
+								    }
+								  }
+								  delete pPrevRec;
+								  delete pCurRec;
+								}
+								  
 							}
 							else
 							{
@@ -501,26 +579,17 @@ bool fp_Page::overlapsWrappedFrame(fp_Line * pLine)
 bool fp_Page::overlapsWrappedFrame(UT_Rect & rec)
 {
 	UT_sint32 i=0;
-	for(i=0; i<static_cast<UT_sint32>(countFrameContainers());i++)
+	for(i=0; i<static_cast<UT_sint32>(countAboveFrameContainers());i++)
 	{
-		fp_FrameContainer * pFC = getNthFrameContainer(i);
+		fp_FrameContainer * pFC = getNthAboveFrameContainer(i);
 		if(!pFC->isWrappingSet())
 		{
 			continue;
 		}
-		UT_Rect * pMyFrameRec = pFC->getScreenRect();
-		fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFC->getSectionLayout());
-		UT_sint32 iextra = pFL->getBoundingSpace() -2;
-		pMyFrameRec->left -= iextra;
-		pMyFrameRec->top -= iextra;
-		pMyFrameRec->width += 2*iextra;
-		pMyFrameRec->height += 2*iextra;
-		if(rec.intersectsRect(pMyFrameRec))
+		if(pFC->overlapsRect(rec))
 		{
-			delete pMyFrameRec;
-			return true;
+		        return true;
 		}
-		delete pMyFrameRec;
 	}
 	return false;
 }
@@ -738,9 +807,11 @@ void fp_Page::getScreenOffsets(fp_Container* pContainer, UT_sint32& xoff, UT_sin
 	    return;
 	}
 	m_pView->getPageScreenOffsets(this, xoff, yoff);
-
-	xoff += pContainer->getX();
-	yoff += pContainer->getY();
+	if(pContainer)
+        {
+	    xoff += pContainer->getX();
+	    yoff += pContainer->getY();
+	}
 }
 
 fp_Page* fp_Page::getNext(void) const
@@ -856,6 +927,29 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 
 	UT_sint32 count = 0;
 
+	// draw Below Frames
+	count = m_vecBelowFrames.getItemCount();
+	for (i=0; i<count; i++)
+	{
+		fp_FrameContainer* pFC = m_vecBelowFrames.getNthItem(i);
+		UT_Rect r(pFC->getX(),pFC->getY(),pFC->getWidth(),pFC->getHeight());
+		if(m_rDamageRect.intersectsRect(&r))
+		{
+			pFC->setOverWrote();
+		}
+		dg_DrawArgs da = *pDA;
+		if(m_pView && (m_pView->getViewMode() != VIEW_PRINT) && !pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
+		{
+			fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
+			fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
+			da.yoff -= pFirstSectionLayout->getTopMargin();
+		}
+		da.xoff += pFC->getX();
+		da.yoff += pFC->getY();
+		pFC->draw(&da);
+	}
+
+
 	// draw each column on the page
 	count = m_vecColumnLeaders.getItemCount();
 
@@ -924,11 +1018,11 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 		pFC->draw(&da);
 	}
 
-	// draw Frames
-	count = m_vecFrames.getItemCount();
+	// draw Above Frames
+	count = m_vecAboveFrames.getItemCount();
 	for (i=0; i<count; i++)
 	{
-		fp_FrameContainer* pFC = m_vecFrames.getNthItem(i);
+		fp_FrameContainer* pFC = m_vecAboveFrames.getNthItem(i);
 		UT_Rect r(pFC->getX(),pFC->getY(),pFC->getWidth(),pFC->getHeight());
 		if(m_rDamageRect.intersectsRect(&r))
 		{
@@ -981,11 +1075,11 @@ void   fp_Page::expandDamageRect(UT_sint32 x, UT_sint32 y,
 void   fp_Page::redrawDamagedFrames(dg_DrawArgs* pDA)
 {
 	// draw Frames
-	UT_sint32 count = m_vecFrames.getItemCount();
+	UT_sint32 count = m_vecAboveFrames.getItemCount();
 	UT_sint32 i = 0;
 	for (i=0; i<count; i++)
 	{
-		fp_FrameContainer* pFC = m_vecFrames.getNthItem(i);
+		fp_FrameContainer* pFC = m_vecAboveFrames.getNthItem(i);
 		UT_Rect r(pFC->getX(),pFC->getY(),pFC->getWidth(),pFC->getHeight());
 		if(m_rDamageRect.intersectsRect(&r))
 		{
@@ -1799,9 +1893,9 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 		// loop from high z to low z
 		// Because we draw from old to new, the new appears on top
 		// and because the new appears on top, it should be treated as such
-		for (i=(static_cast<UT_sint32>(countFrameContainers()-1)); i>=0; i--)
+		for (i=(static_cast<UT_sint32>(countAboveFrameContainers()-1)); i>=0; i--)
 		{
-			pFrameC = getNthFrameContainer(i);
+			pFrameC = getNthAboveFrameContainer(i);
 			bool isImage = false;
 			fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFrameC->getSectionLayout());
 			if(pFL->getFrameType() >= FL_FRAME_WRAPPER_IMAGE)
@@ -1856,6 +1950,64 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 				}
 			}
 		}
+		for (i=(static_cast<UT_sint32>(countBelowFrameContainers()-1)); i>=0; i--)
+		{
+			pFrameC = getNthBelowFrameContainer(i);
+			bool isImage = false;
+			fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFrameC->getSectionLayout());
+			if(pFL->getFrameType() >= FL_FRAME_WRAPPER_IMAGE)
+			{
+				isImage = true;
+			}
+			if ((pFrameC->getFirstContainer()) || isImage )
+			{
+				if ((x >= (pFrameC->getFullX()- iextra))
+					&& (x < (pFrameC->getFullX() + pFrameC->getFullWidth()+iextra))
+					&& (y >= (pFrameC->getFullY() - iextra))
+					&& (y < (pFrameC->getFullY() + pFrameC->getFullHeight() + iextra))
+					)
+				{
+					if(isImage)
+					{
+						pos = pFL->getPosition(true);
+						return;
+					}
+					pFrameC->mapXYToPosition(x - pFrameC->getX(), y - pFrameC->getY(), pos, bBOL, bEOL,isTOC);
+					return;
+				}
+				
+				iDist = pFrameC->distanceFromPoint(x, y);
+//
+// The tlu(3) makes the distance of the mouse to the sensitive edge of the
+// text box 3 pixels. ie Move the mouse within 3 pixels of the textbox and it
+// change to show you can select the text box.
+//
+// If we're outside this distance make sure all other options are excluded
+// before placing hte point inside th text box
+//
+
+				if(static_cast<UT_sint32>(iDist) > m_pLayout->getGraphics()->tlu(3))
+				{
+					iDist += 200000;
+				}
+				if (iDist < iMinDist)
+				{
+					iMinDist = iDist;
+					pMinDist = static_cast<fp_VerticalContainer *>(pFrameC);
+				}
+				
+				if ( (y >= pFrameC->getY())
+					 && (y < (pFrameC->getY() + pFrameC->getHeight()))) 
+				{
+					if (iDist < iMinXDist)
+					{
+						iMinXDist = iDist;
+						pMinXDist = static_cast<fp_VerticalContainer *>(pFrameC);
+					}
+				}
+			}
+		}
+
 	}
 //
 // Look in header for insertion point
@@ -2102,9 +2254,13 @@ void fp_Page::frameHeightChanged(void)
 void fp_Page::clearScreenFrames(void)
 {
 	UT_sint32 i =0;
-	for (i = 0; i < static_cast<UT_sint32>(countFrameContainers()); i++)
+	for (i = 0; i < static_cast<UT_sint32>(countAboveFrameContainers()); i++)
 	{
-		getNthFrameContainer(i)->clearScreen();
+		getNthAboveFrameContainer(i)->clearScreen();
+	}
+	for (i = 0; i < static_cast<UT_sint32>(countBelowFrameContainers()); i++)
+	{
+		getNthBelowFrameContainer(i)->clearScreen();
 	}
 }
 
@@ -2153,37 +2309,75 @@ void fp_Page::markDirtyOverlappingRuns(fp_FrameContainer * pFrameC)
 
 	// Now Frames
 
-	count = m_vecFrames.getItemCount();
+	count = m_vecAboveFrames.getItemCount();
 	for (i=0; i<count; i++)
 	{
-		fp_FrameContainer* pFC = m_vecFrames.getNthItem(i);
+		fp_FrameContainer* pFC = m_vecAboveFrames.getNthItem(i);
 		if(pFC != pFrameC)
 		{
 			pFC->markDirtyOverlappingRuns(*pMyFrameRect);
 		}
 	}
-	delete pMyFrameRect;
+
+
+	count = m_vecBelowFrames.getItemCount();
+	for (i=0; i<count; i++)
+	{
+		fp_FrameContainer* pFC = m_vecBelowFrames.getNthItem(i);
+		if(pFC != pFrameC)
+		{
+			pFC->markDirtyOverlappingRuns(*pMyFrameRect);
+		}
+	}
+	DELETEP(pMyFrameRect);
 }
 
-UT_uint32 fp_Page::countFrameContainers(void) const
+
+UT_uint32 fp_Page::countAboveFrameContainers(void) const
 {
-	return m_vecFrames.getItemCount();
+        return m_vecAboveFrames.getItemCount();
+}
+
+
+UT_uint32 fp_Page::countBelowFrameContainers(void) const
+{
+        return m_vecBelowFrames.getItemCount();
 }
 
 UT_sint32 fp_Page::findFrameContainer(fp_FrameContainer * pFC)
 {
-	UT_sint32 i = m_vecFrames.findItem(pFC);
+        UT_sint32 i; 
+        if(pFC->isAbove())
+	{
+	  i = m_vecAboveFrames.findItem(pFC);
+	  return i;
+
+	}
+        i = m_vecBelowFrames.findItem(pFC);
 	return i;
 }
 
-fp_FrameContainer* fp_Page::getNthFrameContainer(UT_sint32 n) const 
+fp_FrameContainer* fp_Page::getNthAboveFrameContainer(UT_sint32 n) const 
 {
-	return m_vecFrames.getNthItem(n);
+	return m_vecAboveFrames.getNthItem(n);
+} 
+
+
+fp_FrameContainer* fp_Page::getNthBelowFrameContainer(UT_sint32 n) const 
+{
+	return m_vecBelowFrames.getNthItem(n);
 } 
 
 bool fp_Page::insertFrameContainer(fp_FrameContainer * pFC)
 {
-	m_vecFrames.addItem(pFC);
+        if(pFC->isAbove())
+	{
+	       m_vecAboveFrames.addItem(pFC);
+	}
+	else
+	{
+	       m_vecBelowFrames.addItem(pFC);
+	}
 	if(pFC)
 	{
 		pFC->setPage(this);
@@ -2196,16 +2390,40 @@ void fp_Page::removeFrameContainer(fp_FrameContainer * pFC)
 {
 
 	markDirtyOverlappingRuns(pFC);
-	UT_sint32 ndx = m_vecFrames.findItem(pFC);
+	UT_sint32 ndx = 0;
+	bool isAbove = false;
+	if(pFC->isAbove())
+	{
+	    ndx = m_vecAboveFrames.findItem(pFC);
+	    isAbove = true;
+	}
+	else
+	{
+	    ndx = m_vecBelowFrames.findItem(pFC);
+	}
 	if(ndx>=0)
 	{
-		m_vecFrames.deleteNthItem(ndx);
-		for(ndx=0; ndx < static_cast<UT_sint32>(countFrameContainers());ndx++)
-		{			
-			fp_FrameContainer * pFC = getNthFrameContainer(ndx);
-			fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFC->getSectionLayout());
-			pFC->clearScreen();
-			pFL->markAllRunsDirty();
+	        if(isAbove)
+		{
+		        m_vecAboveFrames.deleteNthItem(ndx);
+			for(ndx=0; ndx < static_cast<UT_sint32>(countAboveFrameContainers());ndx++)
+			{			
+			    fp_FrameContainer * pFC = getNthAboveFrameContainer(ndx);
+			    fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFC->getSectionLayout());
+			    pFC->clearScreen();
+			    pFL->markAllRunsDirty();
+			}
+		}
+		else
+		{
+		        m_vecBelowFrames.deleteNthItem(ndx);
+			for(ndx=0; ndx < static_cast<UT_sint32>(countAboveFrameContainers());ndx++)
+			{			
+			    fp_FrameContainer * pFC = getNthAboveFrameContainer(ndx);
+			    fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFC->getSectionLayout());
+			    pFC->clearScreen();
+			    pFL->markAllRunsDirty();
+			}
 		}
 		_reformat();
 		return;
