@@ -54,7 +54,9 @@ pt_PieceTable::pt_PieceTable(PD_Document * pDocument)
 	m_pDocument(pDocument),
     m_atomicGlobCount(0),
 	m_bDoingTheDo(false),
-	m_bDoNotTweakPosition(false)
+	m_bDoNotTweakPosition(false),
+	m_iXID(0),
+	m_iCurCRNumber(0)
 {
 
 	setPieceTableState(PTS_Create);
@@ -244,8 +246,11 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 	const pf_Frag * pf = static_cast<const pf_Frag *>(sdh);
 	UT_return_val_if_fail (pf->getType() == pf_Frag::PFT_Strux,false);
 	const pf_Frag_Strux * pfsBlock = static_cast<const pf_Frag_Strux *> (pf);
-	UT_return_val_if_fail (pfsBlock->getStruxType() == PTX_Block,false);
-
+	
+	// This assert is incorrect; blocks that are inserted inside a TOC use sdh of the TOC section
+	// UT_return_val_if_fail (pfsBlock->getStruxType() == PTX_Block,false);
+	UT_return_val_if_fail (pfsBlock->getStruxType() == PTX_Block || pfsBlock->getStruxType() == PTX_SectionTOC,false);
+	
 	UT_uint32 cumOffset = 0;
 	UT_uint32 cumEndOffset = 0;
 	pf_Frag * pfTemp = NULL;
@@ -1126,7 +1131,7 @@ bool pt_PieceTable::_changePointWithNotify(PT_DocPosition dpos)
 {
 	PX_ChangeRecord * pcr
 		= new PX_ChangeRecord(PX_ChangeRecord::PXT_ChangePoint,
-							  dpos, 0);
+							  dpos, 0,0);
 	UT_return_val_if_fail (pcr,false);
 
 	m_history.addChangeRecord(pcr);
@@ -1135,5 +1140,26 @@ bool pt_PieceTable::_changePointWithNotify(PT_DocPosition dpos)
 	return true;
 }
 
+/*!
+    This function crawls the entire PT and assignes new xid to any fragment that should
+    have one and does not. It is primarily to be used by exporters (accessed throught
+    PD_Document wrapper)
+*/
+void pt_PieceTable::fixMissingXIDs()
+{
+	for (pf_Frag * pf = m_fragments.getFirst(); (pf); pf=pf->getNext())
+	{
+		if(!pf->getXID() && pf->usesXID())
+			pf->setXID(getXID());
+	}
+}
+
+UT_uint32 pt_PieceTable::getXID()
+{
+	++m_iXID;
+	return m_iXID;
+}
+
+
 
 
