@@ -207,6 +207,7 @@ bool AP_Win32App::initialize(void)
 			{
 				pDiskStringSet->setFallbackStringSet(m_pStringSet);
 				m_pStringSet = pDiskStringSet;
+                UT_Language_updateLanguageNames();
 				UT_DEBUGMSG(("Using StringSet [%s]\n",szPathname));
 			}
 			else
@@ -364,11 +365,6 @@ bool AP_Win32App::initialize(void)
 	}
 	_findclose( findtag );
 
-	/* SPI modules don't register automatically on loading, so
-	 * now that we've loaded the modules we need to register them:
-	 */
-	XAP_ModuleManager::instance().registerPending ();
-
 	return bSuccess;
 }
 
@@ -820,57 +816,49 @@ PBITMAPINFO CreateBitmapInfoStruct(HBITMAP hBmp)
 //
 //
 //
-void CreateBMPFile(HWND hwnd, LPTSTR pszFile, PBITMAPINFO pbi, 
+void CreateBMP(HWND hwnd, UT_ByteBuf & pBB, PBITMAPINFO pbi, 
                   HBITMAP hBMP, HDC hDC) 
 { 
-	HANDLE hf;                 // file handle 
 	BITMAPFILEHEADER hdr;       // bitmap file-header 
 	PBITMAPINFOHEADER pbih;     // bitmap info-header 
 	LPBYTE lpBits;              // memory pointer 	
-//	DWORD cb;                   // incremental count of bytes - not used
-	DWORD dwTmp; 
 
-    pbih = (PBITMAPINFOHEADER) pbi; 
-    lpBits = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbih->biSizeImage);
+	if (!hBMP) return;
 
-    if (!lpBits) return;
+	pbih = (PBITMAPINFOHEADER) pbi; 
+	lpBits = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbih->biSizeImage);
 
-    // Retrieve the color table (RGBQUAD array) and the bits 
-    // (array of palette indices) from the DIB. 
-    if (!GetDIBits(hDC, hBMP, 0, (WORD) pbih->biHeight, lpBits, pbi, 
-        DIB_RGB_COLORS)) 
-    	return;
+	if (!lpBits) return;
 
-    // Create the .BMP file. 
-    hf = CreateFile(pszFile,  GENERIC_READ | GENERIC_WRITE, 
-                   (DWORD) 0,  NULL,  CREATE_ALWAYS,  FILE_ATTRIBUTE_NORMAL, (HANDLE) NULL); 
-                   
-    if (hf == INVALID_HANDLE_VALUE)  return;
-        
-    hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M" 
-    // Compute the size of the entire file. 
-    hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) + 
-                 pbih->biSize + pbih->biClrUsed 
-                 * sizeof(RGBQUAD) + pbih->biSizeImage); 
-    hdr.bfReserved1 = 0; 
-    hdr.bfReserved2 = 0; 
+	// Retrieve the color table (RGBQUAD array) and the bits 
+	// (array of palette indices) from the DIB. 
+	if (!GetDIBits(hDC, hBMP, 0, (WORD) pbih->biHeight, lpBits, pbi, 
+		 DIB_RGB_COLORS)) 
+	return;
 
-    // Compute the offset to the array of color indices. 
-    hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + 
-                    pbih->biSize + pbih->biClrUsed 
-                    * sizeof (RGBQUAD); 
+	hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M" 
+	// Compute the size of the entire file. 
+	hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) + 
+			pbih->biSize + pbih->biClrUsed 
+			* sizeof(RGBQUAD) + pbih->biSizeImage); 
+	hdr.bfReserved1 = 0; 
+	hdr.bfReserved2 = 0; 
 
-    // Copy the BITMAPFILEHEADER into the .BMP file. 
-    WriteFile(hf, (LPVOID) &hdr, sizeof(BITMAPFILEHEADER),  (LPDWORD) &dwTmp,  NULL);
-    
-    WriteFile(hf, (LPVOID) pbih, sizeof(BITMAPINFOHEADER) + pbih->biClrUsed * sizeof (RGBQUAD), 
-                  (LPDWORD) &dwTmp, ( NULL));        
+	// Compute the offset to the array of color indices. 
+	hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + 
+	pbih->biSize + pbih->biClrUsed 
+	* sizeof (RGBQUAD); 
 
-    // Copy the array of color indices into the .BMP file.         
-    WriteFile(hf, (LPSTR) lpBits, (int) pbih->biSizeImage, (LPDWORD) &dwTmp,NULL);           
-    
-	CloseHandle(hf);        
-    GlobalFree((HGLOBAL)lpBits);
+	pBB.truncate (0);
+
+	// Copy the BITMAPFILEHEADER into the .BMP file. 
+	pBB.append ((const UT_Byte *)&hdr, sizeof(BITMAPFILEHEADER));
+	pBB.append ((const UT_Byte *)pbih, sizeof(BITMAPINFOHEADER) + pbih->biClrUsed * sizeof (RGBQUAD));
+
+	// Copy the array of color indices into the .BMP file.         
+	pBB.append ((const UT_Byte *)lpBits, (int) pbih->biSizeImage);
+
+	GlobalFree((HGLOBAL)lpBits);
 }
 
 
@@ -940,28 +928,23 @@ bool AP_Win32App::_pasteFormatFromClipboard(PD_DocumentRange * pDocRange, const
  		AP_FrameData* 			pFrameData;		
  		FL_DocLayout*			pDocLy;	
  		FV_View* 				pView;						
- 		char szFile[_MAX_PATH];
- 		char szPath[MAX_PATH];
+		UT_ByteBuf				bBufBMP;
  		
  		hBitmap = (HBITMAP)hData;					
  		hWnd =  GetDesktopWindow();
  		hdc = GetDC(hWnd);		
  		
- 		// Get a temp file
- 		GetTempPath(MAX_PATH,szPath);
-  		GetTempFileName(szPath, "abi", rand()*65535*65535, szFile);		
- 
-  		// Create a BMP file from a BITMAP
+ 		// Create a BMP file from a BITMAP
  		bi =  CreateBitmapInfoStruct(hBitmap);						
- 		CreateBMPFile(hWnd, szFile, bi, hBitmap,hdc);                  										
+ 		CreateBMP(hWnd, bBufBMP, bi, hBitmap,hdc);                  										
  		
  		// Since we are providing the file type, there is not need to pass the bytebuff filled up
- 		errorCode = IE_ImpGraphic::constructImporter(szFile, iegft, &pIEG);				 				
+ 		errorCode = IE_ImpGraphic::constructImporter(&bBufBMP, iegft, &pIEG);				 				
 		 				
  		if(errorCode != UT_OK)		
 			return false;				  	
 		 				 			
- 		errorCode = pIEG->importGraphic(szFile, &pFG); 		
+ 		errorCode = pIEG->importGraphic(&bBufBMP, &pFG); 		
  		
  		if(errorCode != UT_OK || !pFG)
 		{
@@ -979,8 +962,7 @@ bool AP_Win32App::_pasteFormatFromClipboard(PD_DocumentRange * pDocRange, const
  	
 		DELETEP(pIEG);
  		//DELETEP(pFG);		
-
- 		unlink(szFile);		  	
+ 		
  		bSuccess = true;
  	}
  	else	
@@ -1134,7 +1116,7 @@ static LRESULT CALLBACK _SplashWndProc(HWND hWnd, UINT message, WPARAM wParam, L
         break;
         
     default:
-        return (DefWindowProc(hWnd, message, wParam, lParam));
+        return (UT_DefWindowProc(hWnd, message, wParam, lParam));
     }
     return (0);
 }
@@ -1157,26 +1139,15 @@ static GR_Image * _showSplash(HINSTANCE hInstance, const char * szAppName)
 		)
 	{
 		// NB: can't access 'this' members from a static member function
-		WNDCLASSEX  wndclass;
 		ATOM a;
 	
 		sprintf(s_SplashWndClassName, "%sSplash", szAppName /* app->getApplicationName() */);
 
-		// register class for the splash window
-		wndclass.cbSize        = sizeof(wndclass);
-		wndclass.style         = 0;
-		wndclass.lpfnWndProc   = _SplashWndProc;
-		wndclass.cbClsExtra    = 0;
-		wndclass.cbWndExtra    = 0;
-		wndclass.hInstance     = hInstance /* app->getInstance() */;
-		wndclass.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_32)) /* app->getIcon() */;
-		wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
-		wndclass.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
-		wndclass.lpszMenuName  = NULL;
-		wndclass.lpszClassName = s_SplashWndClassName;
-		wndclass.hIconSm       = LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_16)) /* app->getSmallIcon() */;
-
-		a = RegisterClassEx(&wndclass);
+		a = UT_RegisterClassEx(0L, _SplashWndProc, hInstance,
+							   LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_32)),
+							   LoadCursor(NULL, IDC_ARROW), (HBRUSH) GetStockObject(NULL_BRUSH),
+							   LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_16)),
+							   NULL, s_SplashWndClassName);
 		UT_ASSERT_HARMLESS(a);
 
 		// get the extents of the desktop window
@@ -1189,7 +1160,7 @@ static GR_Image * _showSplash(HINSTANCE hInstance, const char * szAppName)
 		UT_PNG_getDimensions(pBB, iSplashWidth, iSplashHeight);
 
 		// create a centered window the size of our bitmap
-		hwndSplash = CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST,s_SplashWndClassName, 
+		hwndSplash = UT_CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST,s_SplashWndClassName, 
 								  NULL, WS_POPUP | WS_BORDER,
 								  (rect.right  / 2) - (iSplashWidth  / 2) - 1, // subtract 1 pixel to account for WS_BORDER
 								  (rect.bottom / 2) - (iSplashHeight / 2) - 1, // subtract 1 pixel to account for WS_BORDER
@@ -1346,7 +1317,7 @@ try
 		}	
 
 		// do dispatch loop
-		while( GetMessage(&msg, NULL, 0, 0) )
+		while(UT_GetMessage(&msg, NULL, 0, 0))
 	    {
    	      	// TranslateMessage is not called because AbiWord
 	      	// has its own way of decoding keyboard accelerators
@@ -1354,7 +1325,7 @@ try
 				continue;
 				
 			TranslateMessage(&msg);	
-	    	DispatchMessage(&msg);	
+			UT_DispatchMessage(&msg);
 	    	
 			// Check for idle condition
 			while( !UT_Win32Idle::_isEmpty() &&
@@ -1585,7 +1556,7 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args)
 			}
 			
 			conv.setVerbose(Args->m_iVerbose);
-			conv.print (Args->m_sFile, pG);
+			conv.print (Args->m_sFile, pG, Args->m_sFileExtension);
 	      
 			delete pG;
 		}
@@ -1709,26 +1680,25 @@ bool	AP_Win32App::doesStringSetExist(const char* pLocale)
 	return false;
 }
 
-/* From UCS4 To Ansi*/
-UT_String  	AP_Win32App::s_fromUCS4ToAnsi(const UT_UCS4Char * szIn)
-{	
-	UT_uint32 uRead, uWrite;
+/* From UCS4 To WinLocale */
+UT_String  	AP_Win32App::s_fromUCS4ToWinLocale(const UT_UCS4Char * szIn)
+{		
 	UT_UCS4String sUCS4(szIn);
 	UT_String sRslt;
 	
 	char *pText = UT_convert ((char *)sUCS4.ucs4_str(),
 							  sUCS4.length()*sizeof(UT_UCS4Char),
 							  ucs4Internal(),
-							  XAP_EncodingManager::get_instance()->getNative8BitEncodingName(),
-							  &uRead, &uWrite);
+							  XAP_App::getApp()->getDefaultEncoding(),
+							  NULL, NULL);
 	sRslt = pText;
 	free(pText);
 	return sRslt;
 
 }
 
-/* From  Ansi To UCS4*/
-UT_UCS4String	AP_Win32App::s_fromAnsiToUCS4(const char* szIn)
+/* From WinLocale To UCS4*/
+UT_UCS4String	AP_Win32App::s_fromWinLocaleToUCS4(const char* szIn)
 {
 	UT_UCS4Char * src = new UT_UCS4Char[strlen(szIn)+1];	
 	UT_UCS4_strcpy_char(src, (char*)szIn);	
@@ -1738,19 +1708,30 @@ UT_UCS4String	AP_Win32App::s_fromAnsiToUCS4(const char* szIn)
 	return sRslt;
 }
 
-/* From  UTF8 To Ansi*/
-UT_String 		AP_Win32App::s_fromUTF8ToAnsi(const char* szInUTF8)
+/* From  UTF8 To WinLocale */
+UT_String 		AP_Win32App::s_fromUTF8ToWinLocale(const char* szInUTF8)
 {
 	UT_UTF8String utf8(szInUTF8);	
 	UT_UCS4String sUCS4(utf8.ucs4_str());
-	return AP_Win32App::s_fromUCS4ToAnsi(sUCS4.ucs4_str());
+	return AP_Win32App::s_fromUCS4ToWinLocale(sUCS4.ucs4_str());
 }
 
-/* From  Ansi To UTF8*/
-UT_UTF8String	AP_Win32App::s_fromAnsiToUTF8(const char* szIn)
+/* From WinLocale To UTF8*/
+UT_UTF8String	AP_Win32App::s_fromWinLocaleToUTF8(const char* szIn)
 {
-	UT_UCS4String sUCS4 = AP_Win32App::s_fromAnsiToUCS4(szIn);
+	UT_UCS4String sUCS4 = AP_Win32App::s_fromWinLocaleToUCS4(szIn);
 	UT_UTF8String sRslt(sUCS4.utf8_str());	
 
 	return sRslt;
 }
+
+GR_Graphics * AP_Win32App::newDefaultScreenGraphics() const
+{
+	XAP_Frame * pFrame = findValidFrame();
+	UT_return_val_if_fail( pFrame, NULL );
+	
+	AP_Win32FrameImpl * pFI = (AP_Win32FrameImpl *) pFrame->getFrameImpl();
+	UT_return_val_if_fail( pFI, NULL );
+
+	return pFI->createDocWndGraphics();
+}
