@@ -75,7 +75,9 @@ AP_Dialog_Lists::AP_Dialog_Lists(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id
 	m_pFakeAuto(0),
 	m_pFakeDoc(0),
 	m_bDirty(false),
-    m_bIsModal(false)
+	m_bIsModal(false),
+	m_iCurrentLevel(0),
+	m_bFoldingLevelChanged(false)
 {
 	for(UT_uint32 i=0; i<4; i++)
 	{
@@ -119,9 +121,9 @@ AP_Dialog_Lists::tAnswer AP_Dialog_Lists::getAnswer(void) const
 
 /*!
  * Create the preview from the Graphics Context provided by the platform code.
- \params gc the Platform Graphics Context cast into the a XP handle.
- \params width the width of the gc
- \params height the height of the gc
+ \param gc the Platform Graphics Context cast into the a XP handle.
+ \param width the width of the gc
+ \param height the height of the gc
  */
 void AP_Dialog_Lists::_createPreviewFromGC(GR_Graphics* gc,
 										   UT_uint32 width,
@@ -223,28 +225,47 @@ void AP_Dialog_Lists::setTick(UT_uint32 iTick)
 void AP_Dialog_Lists::Apply(void)
 {
 	XML_Char szStart[20];
-	if(!isModal() && !isPageLists())
+	if(!isModal() && (!isPageLists() || m_bFoldingLevelChanged))
 	{
 //
 // OK fold up the text according the level specified.
 //
+	        m_bFoldingLevelChanged = false;
 		fl_AutoNum * pAuto = getBlock()->getAutoNum();
-		if(!pAuto)
-		{
-			return;
-		}
 		const XML_Char * props[5] = {"text-folded",NULL,"text-folded-id",NULL,NULL};
 		UT_UTF8String sStr = UT_UTF8String_sprintf("%d",getCurrentFold());
 		props[1] = sStr.utf8_str();
-		PT_DocPosition posLow = 0;
-		PT_DocPosition posHigh = 0;
-		UT_uint32 ID = pAuto->getID();
+		UT_uint32 ID = 0;
+		if(!pAuto)
+		{
+		        ID = getView()->getDocument()->getUID(UT_UniqueId::List);
+		}
+		else
+		{
+		        ID = pAuto->getID();
+		}
 		UT_UTF8String sID = UT_UTF8String_sprintf("%d",ID);
 		props[3] = sID.utf8_str();
-		PL_StruxDocHandle sdhLow = pAuto->getFirstItem();
-		PL_StruxDocHandle sdhHigh = pAuto->getLastItem();
-		posLow = getView()->getDocument()->getStruxPosition(sdhLow)+1;
-		posHigh = getView()->getDocument()->getStruxPosition(sdhHigh)+1;
+		PT_DocPosition posLow = 0;
+		PT_DocPosition posHigh = 0;
+		if(getView()->isSelectionEmpty() && pAuto)
+		{
+		      PL_StruxDocHandle sdhLow = pAuto->getFirstItem();
+		      PL_StruxDocHandle sdhHigh = pAuto->getLastItemInHeiracy();
+		      posLow = getView()->getDocument()->getStruxPosition(sdhLow)+1;
+		      posHigh = getView()->getDocument()->getStruxPosition(sdhHigh)+1;
+		}
+		else
+		{
+		      posLow = getView()->getPoint();
+		      posHigh = getView()->getSelectionAnchor();
+		      if(posLow > posHigh)
+		      {
+			  PT_DocPosition posTemp = posLow;
+			  posLow = posHigh;
+			  posHigh = posTemp;
+		      }
+		}
 		getView()->setCollapsedRange(posLow,posHigh,props);
 		return;
 	}
