@@ -40,7 +40,9 @@
 
 menuitem(FILE)
 menuitem(FILE_NEW)
+menuitem(FILE_NEW_USING_TEMPLATE)
 menuitem(FILE_OPEN)
+menuitem(FILE_IMPORTSTYLES)
 menuitem(FILE_SAVE)
 menuitem(FILE_SAVEAS)
 menuitem(FILE_SAVE_TEMPLATE)
@@ -65,6 +67,7 @@ menuitem(FILE_RECENT_8)
 menuitem(FILE_RECENT_9)
 menuitem(FILE_REVERT)
 menuitem(FILE_EXIT)
+menuitem(FILE_SAVEEMBED)
 
 menuitem(OPEN_TEMPLATE)
 
@@ -84,13 +87,23 @@ menuitem(EDIT_EDITHEADER)
 menuitem(EDIT_EDITFOOTER)
 menuitem(EDIT_REMOVEHEADER)
 menuitem(EDIT_REMOVEFOOTER)
+menuitem(EDIT_DELETEFRAME)
+menuitem(EDIT_CUT_FRAME)
+menuitem(EDIT_COPY_FRAME)
+menuitem(EDIT_SELECT_FRAME)
+menuitem(EDIT_DELETEIMAGE)
+menuitem(EDIT_CUTIMAGE)
+menuitem(EDIT_COPYIMAGE)
+menuitem(EDIT_CUTEMBED)
+menuitem(EDIT_COPYEMBED)
+menuitem(EDIT_DELETEEMBED)
 
 menuitem(VIEW)
 menuitem(VIEW_TOOLBARS)
-menuitem(VIEW_TB_STD)
-menuitem(VIEW_TB_FORMAT)
-menuitem(VIEW_TB_EXTRA)
-menuitem(VIEW_TB_TABLE)
+menuitem(VIEW_TB_1)
+menuitem(VIEW_TB_2)
+menuitem(VIEW_TB_3)
+menuitem(VIEW_TB_4)
 menuitem(VIEW_LOCK_TB_LAYOUT)
 menuitem(VIEW_DEFAULT_TB_LAYOUT)
 menuitem(VIEW_RULER)
@@ -117,6 +130,7 @@ menuitem(INSERT_BREAK)
 menuitem(INSERT_PAGENO)
 menuitem(INSERT_DATETIME)
 menuitem(INSERT_FIELD)
+menuitem(INSERT_TEXTBOX)
 menuitem(INSERT_MAILMERGE)
 menuitem(INSERT_FILE)
 menuitem(INSERT_SYMBOL)
@@ -126,10 +140,14 @@ menuitem(INSERT_PICTURE)
 menuitem(INSERT_CLIPART)
 menuitem(INSERT_GRAPHIC)
 menuitem(INSERT_BOOKMARK)
+menuitem(INSERT_TABLEOFCONTENTS)
 menuitem(INSERT_HYPERLINK)
+menuitem(INSERT_GOTO_HYPERLINK)
+menuitem(INSERT_EDIT_HYPERLINK)
 menuitem(INSERT_DELETE_HYPERLINK)
-menuitem(INSERT_INSERTHEADER)
-menuitem(INSERT_INSERTFOOTER)
+menuitem(EDIT_COPY_HYPERLINK_LOCATION)
+menuitem(INSERT_HEADER)
+menuitem(INSERT_FOOTER)
 menuitem(INSERT_DIRECTIONMARKER)	
 menuitem(INSERT_DIRECTIONMARKER_LRM)	
 menuitem(INSERT_DIRECTIONMARKER_RLM)	
@@ -143,15 +161,19 @@ menuitem(FMT_BORDERS)
 menuitem(FMT_COLUMNS)
 menuitem(FMT_TOGGLECASE)
 menuitem(FMT_BACKGROUND)
+menuitem(FMT_BACKGROUND_PAGE_IMAGE)
+menuitem(FMT_BACKGROUND_PAGE_COLOR)
 menuitem(FMT_HDRFTR)
 menuitem(FMT_FOOTNOTES)
+menuitem(FMT_TABLEOFCONTENTS)
 menuitem(FMT_TABS)
 menuitem(FMT_LANGUAGE)
 menuitem(FMT_IMAGE)
+menuitem(FMT_POSIMAGE)
 menuitem(FMT_STYLE)
 menuitem(FMT_STYLE_DEFINE)
-menuitem(FMT_IMPORTSTYLES)
-
+menuitem(FMT_STYLIST)
+menuitem(FMT_FRAME)
 menuitem(FMT)
 menuitem(FMT_BOLD)
 menuitem(FMT_ITALIC)
@@ -166,6 +188,7 @@ menuitem(FMT_DIRECTION)
 menuitem(FMT_DIRECTION_DD_RTL)
 menuitem(FMT_DIRECTION_DO_LTR)
 menuitem(FMT_DIRECTION_DO_RTL)
+menuitem(FMT_EMBED)
 
 menuitem(TOOLS)
 menuitem(TOOLS_SPELLING)
@@ -180,11 +203,27 @@ menuitem(TOOLS_OPTIONS)
 menuitem(TOOLS_LANGUAGE)
 menuitem(TOOLS_REVISIONS)
 menuitem(TOOLS_REVISIONS_MARK)
+menuitem(TOOLS_REVISIONS_NEW_REVISION)
+menuitem(TOOLS_REVISIONS_SHOW)
+menuitem(TOOLS_REVISIONS_SHOW_AFTER)
+menuitem(TOOLS_REVISIONS_SHOW_AFTERPREV)
+menuitem(TOOLS_REVISIONS_SHOW_BEFORE)
 menuitem(TOOLS_REVISIONS_SET_VIEW_LEVEL)
 menuitem(TOOLS_REVISIONS_ACCEPT_REVISION)
 menuitem(TOOLS_REVISIONS_REJECT_REVISION)
+menuitem(TOOLS_REVISIONS_PURGE)
+menuitem(TOOLS_REVISIONS_COMPARE_DOCUMENTS)
+menuitem(TOOLS_REVISIONS_MERGE_DOCUMENTS)
+menuitem(TOOLS_REVISIONS_FIND_NEXT)
+menuitem(TOOLS_REVISIONS_FIND_PREV)
+menuitem(TOOLS_REVISIONS_AUTO)
+menuitem(TOOLS_HISTORY)
+menuitem(TOOLS_HISTORY_SHOW)
+menuitem(TOOLS_HISTORY_PURGE)
 menuitem(CONTEXT_REVISIONS_ACCEPT_REVISION)
 menuitem(CONTEXT_REVISIONS_REJECT_REVISION)
+menuitem(CONTEXT_REVISIONS_FIND_NEXT)
+menuitem(CONTEXT_REVISIONS_FIND_PREV)
 menuitem(ALIGN)
 menuitem(ALIGN_LEFT)
 menuitem(ALIGN_CENTER)
@@ -203,6 +242,8 @@ menuitem(TABLE_INSERT_ROWS_BEFORE)
 menuitem(TABLE_INSERT_ROWS_AFTER)
 menuitem(TABLE_INSERTROW)
 menuitem(TABLE_INSERT_CELLS)
+menuitem(TABLE_INSERT_SUMCOLS)
+menuitem(TABLE_INSERT_SUMROWS)
 
 menuitem(TABLE_DELETE)
 menuitem(TABLE_DELETE_TABLE)
@@ -223,8 +264,21 @@ menuitem(TABLE_MERGE_CELLS)
 menuitem(TABLE_SPLIT_CELLS)
 menuitem(TABLE_SPLIT_TABLE)
 menuitem(TABLE_FORMAT)
+menuitem(TABLE_TEXTTOTABLE)
+menuitem(TABLE_TEXTTOTABLE_ALL)
+menuitem(TABLE_TEXTTOTABLE_NOSPACES)
+menuitem(TABLE_TABLETOTEXT)
+menuitem(TABLE_TABLETOTEXTCOMMAS)
+menuitem(TABLE_TABLETOTEXTTABS)
+menuitem(TABLE_TABLETOTEXTCOMMASTABS)
+menuitem(TABLE_SORTROWSASCEND)
+menuitem(TABLE_SORTROWSDESCEND)
+menuitem(TABLE_SORTCOLSASCEND)
+menuitem(TABLE_SORTCOLSDESCEND)
 menuitem(TABLE_AUTOFIT)
 menuitem(TABLE_HEADING_ROWS_REPEAT)
+menuitem(TABLE_HEADING_ROWS_REPEAT_THIS)
+menuitem(TABLE_HEADING_ROWS_REPEAT_REMOVE)
 menuitem(TABLE_SORT)
 
 menuitem(WINDOW)
@@ -250,8 +304,6 @@ menuitem(HELP_INDEX)
 menuitem(HELP_CHECKVER)
 menuitem(HELP_SEARCH)
 menuitem(HELP_ABOUT)
-menuitem(HELP_ABOUTOS)
-menuitem(HELP_ABOUT_GNU)
 menuitem(HELP_ABOUT_GNOMEOFFICE)
 menuitem(HELP_REPORT_BUG)
 
@@ -267,59 +319,4 @@ menuitem(SPELL_SUGGEST_9)
 menuitem(SPELL_IGNOREALL)
 menuitem(SPELL_ADD)
 
-	/* the following entries are for autotext */
-menuitem(INSERT_AUTOTEXT)
-menuitem(AUTOTEXT_ATTN)
-menuitem(AUTOTEXT_CLOSING)
-menuitem(AUTOTEXT_MAIL)
-menuitem(AUTOTEXT_REFERENCE)
-menuitem(AUTOTEXT_SALUTATION)
-menuitem(AUTOTEXT_SUBJECT)
-menuitem(AUTOTEXT_EMAIL)
-
-menuitem(AUTOTEXT_ATTN_1)
-menuitem(AUTOTEXT_ATTN_2)
-
-menuitem(AUTOTEXT_CLOSING_1)
-menuitem(AUTOTEXT_CLOSING_2)
-menuitem(AUTOTEXT_CLOSING_3)
-menuitem(AUTOTEXT_CLOSING_4)
-menuitem(AUTOTEXT_CLOSING_5)
-menuitem(AUTOTEXT_CLOSING_6)
-menuitem(AUTOTEXT_CLOSING_7)
-menuitem(AUTOTEXT_CLOSING_8)
-menuitem(AUTOTEXT_CLOSING_9)
-menuitem(AUTOTEXT_CLOSING_10)
-menuitem(AUTOTEXT_CLOSING_11)
-menuitem(AUTOTEXT_CLOSING_12)
-
-menuitem(AUTOTEXT_MAIL_1)
-menuitem(AUTOTEXT_MAIL_2)
-menuitem(AUTOTEXT_MAIL_3)
-menuitem(AUTOTEXT_MAIL_4)
-menuitem(AUTOTEXT_MAIL_5)
-menuitem(AUTOTEXT_MAIL_6)
-menuitem(AUTOTEXT_MAIL_7)
-menuitem(AUTOTEXT_MAIL_8)
-
-menuitem(AUTOTEXT_REFERENCE_1)
-menuitem(AUTOTEXT_REFERENCE_2)
-menuitem(AUTOTEXT_REFERENCE_3)
-
-menuitem(AUTOTEXT_SALUTATION_1)
-menuitem(AUTOTEXT_SALUTATION_2)
-menuitem(AUTOTEXT_SALUTATION_3)
-menuitem(AUTOTEXT_SALUTATION_4)
-
-menuitem(AUTOTEXT_SUBJECT_1)
-
-menuitem(AUTOTEXT_EMAIL_1)
-menuitem(AUTOTEXT_EMAIL_2)
-menuitem(AUTOTEXT_EMAIL_3)
-menuitem(AUTOTEXT_EMAIL_4)
-menuitem(AUTOTEXT_EMAIL_5)
-menuitem(AUTOTEXT_EMAIL_6)
-
 /**** Add Menu Items here ****/
-
-
