@@ -1,5 +1,6 @@
 /* AbiSource Program Utilities
  * Copyright (C) 2002-2003 Dom Lachowicz <cinamod@hotmail.com>
+ * Copyright (C) 2005 Robert Staudinger <robsta@stereolyzer.net>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +23,7 @@
 #include <gsf/gsf-infile-zip.h>
 
 #include <locale.h>
+#include <math.h>
 
 #include "ie_impexp_OpenWriter.h"
 #include "ie_impGraphic.h"
@@ -29,6 +31,7 @@
 #include "fg_GraphicRaster.h"
 
 #include "pd_Document.h"
+#include "pd_Style.h"
 
 // abiword stuff
 #include "ut_xml.h"
@@ -42,6 +45,165 @@
 
 #include "ut_debugmsg.h"
 
+class OpenWriter_StylesStream_Listener;
+
+/*****************************************************************************/
+/*****************************************************************************/
+
+/*!
+ * Class representing OOo page setup properties.
+ * Imported values are separated for use within Abi's
+ * <pagesize> and <section> tags.
+ */
+class OO_PageStyle
+{
+public:
+
+  OO_PageStyle() : m_name("")
+  {
+  }
+
+  ~OO_PageStyle()
+  {
+  }
+
+  /*!
+  * Parse attributes array and map keys and values
+  * to Abi's.
+  */
+  void appendPageMaster(const XML_Char * name, const XML_Char ** atts) 
+  {
+    // FIXME ATM only one page setup per document can be imported
+    if (m_name != "")
+    {
+      UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+    }
+
+    m_name = name;
+    parse(atts);
+  }
+
+  /*!
+  * Returns the name of the current page setup style.
+  */
+  const XML_Char * getName() const 
+  {
+    return reinterpret_cast<const XML_Char *>(m_name.c_str());
+  }
+
+  /*!
+  * Returns attribute array for the <pagesize> tag.
+  */
+  const XML_Char ** getAbiPageAtts(const XML_Char * masterName)
+  {
+    UT_return_val_if_fail(masterName != NULL, (const XML_Char **)m_pageAtts);
+
+    if (UT_strcmp (m_name.c_str(), masterName))
+      // FIXME can there be more than one master-page?
+      UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+    
+    return (const XML_Char **)m_pageAtts;
+  }
+  
+  /*!
+  * Returns props string for the <section> tag.
+  */
+  const XML_Char * getAbiSectionProps() const
+  {
+    return reinterpret_cast<const XML_Char *>(m_sectionProps.c_str());
+  }
+  
+private: 
+	
+  void parse (const XML_Char ** props)
+  {
+    const XML_Char * val = NULL;
+    int propCtr = 0;
+    double width = 0;
+    double height = 0;
+    
+    // will go into the <pagesize> tag
+    val = UT_getAttribute ("fo:page-width", props);
+    if (val)
+    {
+      width = rint(UT_convertToDimension(val, DIM_MM));
+      m_width = UT_String_sprintf("%f", width);
+      m_pageAtts[propCtr++] = "width";
+      m_pageAtts[propCtr++] = m_width.c_str();
+    }
+    
+    val = UT_getAttribute ("fo:page-height", props);
+    if (val)
+    {
+      height = rint(UT_convertToDimension(val, DIM_MM));
+      m_height = UT_String_sprintf("%f", height);
+      m_pageAtts[propCtr++] = "height";
+      m_pageAtts[propCtr++] = m_height.c_str();
+    }
+    
+    m_pageAtts[propCtr++] = "units";
+    m_pageAtts[propCtr++] = "mm";    
+    
+    val = UT_getAttribute ("style:print-orientation", props);
+    if (val)
+    {
+      m_pageAtts[propCtr++] = "orientation";
+      m_pageAtts[propCtr++] = strdup (val);
+    }
+    
+    m_pageAtts[propCtr++] = "page-scale";
+    m_pageAtts[propCtr++] = "1.0";    
+        
+    // width and height are rounded to full mm because that's how they are
+    // predefined in Abi and there seem to be rounding errors in oow's exporter
+    fp_PageSize ps(width, height, DIM_MM);
+    m_pageAtts[propCtr++] = "pagetype";
+    m_pageAtts[propCtr++] = ps.getPredefinedName();
+
+    m_pageAtts[propCtr] = 0; 
+
+    // will go as props into the <section> tag
+    val = UT_getAttribute ("fo:margin-left", props);
+    if (val)
+      m_marginLeft = UT_String_sprintf("page-margin-left: %s;", val);
+	
+    val = UT_getAttribute ("fo:margin-top", props);
+    if (val)
+      m_marginTop = UT_String_sprintf("page-margin-top: %s;", val);
+	
+    val = UT_getAttribute ("fo:margin-right", props);
+    if (val)
+      m_marginRight = UT_String_sprintf("page-margin-right: %s;", val);	
+	
+    val = UT_getAttribute ("fo:margin-bottom", props);
+    if (val)
+      m_marginBottom = UT_String_sprintf("page-margin-bottom: %s;", val);
+	
+#define APPEND_STYLE(sty) if (sty.size()) m_sectionProps += sty;
+    APPEND_STYLE(m_marginLeft);
+    APPEND_STYLE(m_marginTop);
+    APPEND_STYLE(m_marginRight);
+    APPEND_STYLE(m_marginBottom);
+#undef APPEND_STYLE
+    if (m_sectionProps.size () > 0)
+      m_sectionProps [m_sectionProps.size()-1] = 0; 
+  }
+  
+  UT_String m_name;
+
+  UT_String m_width;
+  UT_String m_height;
+  
+  UT_String m_marginLeft;
+  UT_String m_marginTop;
+  UT_String m_marginRight;
+  UT_String m_marginBottom;
+  
+  static const int MAX_PAGE_ATTS = 13; // 2*(width height orientation pagetype units page-scale) 0 
+  const XML_Char * m_pageAtts[MAX_PAGE_ATTS];
+  UT_String m_sectionProps;
+}; // class OO_PageStyle
+
 /*****************************************************************************/
 /*****************************************************************************/
 
@@ -50,18 +212,18 @@
  * maps from OO->Abi styles. Used for at least 
  * all Abi section, paragraph, and span styles
  */
-class ABI_EXPORT OO_Style 
+class OO_Style 
 {
 public:
   
-  OO_Style (const XML_Char ** props)
-    : m_bColBreakBefore (false), m_bPageBreakBefore (false)
+  OO_Style (const XML_Char ** props, const PD_Style * pParentStyle, const bool bOpenDocument)
+    : m_bColBreakBefore (false), m_bPageBreakBefore (false), m_pParentStyle (pParentStyle), m_bOpenDocument (bOpenDocument)
   {
     parse (props);
   }
 
-  OO_Style (const OO_Style * other, const XML_Char ** props)
-    : m_bColBreakBefore (false), m_bPageBreakBefore (false)
+  OO_Style (const OO_Style * other, const XML_Char ** props, PD_Style * pParentStyle, const bool bOpenDocument)
+    : m_bColBreakBefore (false), m_bPageBreakBefore (false), m_pParentStyle (pParentStyle), m_bOpenDocument (bOpenDocument)
   {
     if (other)
       *this = *other;
@@ -91,13 +253,10 @@ public:
     return m_bPageBreakBefore;
   }
 
-private:
-
   void parse (const XML_Char ** props)
   {
-    const XML_Char** p = props;   
     const XML_Char * val = NULL;
-    
+
     val = UT_getAttribute ("fo:text-align", props);
     if (val)
       if (!UT_strcmp(val, "end"))
@@ -125,17 +284,38 @@ private:
     if (val)
       m_color = UT_String_sprintf ("color: %s;", val);      
     
-    val = UT_getAttribute ("style:text-background-color", props);
+	if (m_bOpenDocument)
+	    val = UT_getAttribute ("fo:background-color", props);
+	else
+	    val = UT_getAttribute ("style:text-background-color", props);
     if(val)
       m_bgcolor = UT_String_sprintf ("bgcolor: %s;", val);      
     
     val = UT_getAttribute("style:font-name", props);
-    if(val)
-      m_fontName = UT_String_sprintf ("font-face: %s;", val);
-    
+    if(val) 
+      m_fontName = UT_String_sprintf ("font-family: %s;", val);
+
     val = UT_getAttribute("fo:font-size", props);
-    if(val)
-      m_fontSize = UT_String_sprintf ("font-size: %s;", val);
+    if(val) {
+	  UT_Dimension dim = UT_determineDimension(val, DIM_none);
+	  if (dim == DIM_PERCENT && !m_pParentStyle) {
+		UT_DEBUGMSG(("*** [open-writer] no parent style to resolve '%s'\n", val));
+	  }
+      else if (dim == DIM_PERCENT && m_pParentStyle) {
+        // calculate font-size based on parent's 
+        const XML_Char * parentFontSize = NULL;
+        double fontSize = 12;
+		if (m_pParentStyle->getProperty("font-size", parentFontSize)) {
+          fontSize = atoi(parentFontSize) * atoi(val) / 100.0;
+        }
+		else {
+		  UT_DEBUGMSG(("*** [open-writer] using fallback font-size '%f'\n", fontSize));
+		}
+		m_fontSize = UT_String_sprintf ("font-size: %gpt;", rint(fontSize));
+      }
+      else
+        m_fontSize = UT_String_sprintf ("font-size: %s;", val);
+    }
     
     if (UT_getAttribute("fo:language", props) && UT_getAttribute("fo:country", props))
       m_lang = UT_String_sprintf ("lang: %s-%s;", UT_getAttribute("fo:language", props),
@@ -152,35 +332,49 @@ private:
 	m_textPos += "normal;";
     }
 
-    if (UT_getAttribute("style:text-underline", props) || UT_getAttribute("style:text-crossing-out", props))
-      {
-	const XML_Char * undr = UT_getAttribute("style:text-underline", props);
-	const XML_Char * strk = UT_getAttribute("style:text-crossing-out", props);
-	
-	m_textDecoration = "text-decoration: ";
-	
-	if(undr)
-	  if (UT_strcmp(undr, "none") != 0)
-	    m_textDecoration += "underline";
-	
-	if (undr && strk)
-	  m_textDecoration += ",";
-	
-	if(strk)
-	  if (UT_strcmp(strk, "none") != 0)
-	    m_textDecoration += "line-through";
-	
-	  m_textDecoration += ";";
-      }
-
+	const XML_Char * undr = NULL;
+	const XML_Char * strk = NULL;
+	if (m_bOpenDocument) {
+		undr = const_cast<const XML_Char *>(UT_getAttribute("style:text-underline-style", props));
+		strk = const_cast<const XML_Char *>(UT_getAttribute("style:text-line-through-style", props));
+	}
+	else {
+		undr = const_cast<const XML_Char *>(UT_getAttribute("style:text-underline", props));
+		strk = const_cast<const XML_Char *>(UT_getAttribute("style:text-crossing-out", props));
+	}
+	if (undr || strk) {
+		m_textDecoration = "text-decoration: ";
+		
+		if(undr)
+		  if (UT_strcmp(undr, "none") != 0)
+		    m_textDecoration += "underline";
+		
+		if (undr && strk)
+		  m_textDecoration += ",";
+		
+		if(strk)
+		  if (UT_strcmp(strk, "none") != 0)
+		    m_textDecoration += "line-through";
+		
+		  m_textDecoration += ";";
+	}
+ 
     val = UT_getAttribute ("fo:margin-left", props);
     if(val)
       m_marginLeft = UT_String_sprintf ("margin-left: %s;", val);
     
+    val = UT_getAttribute ("fo:margin-top", props);
+    if(val)
+      m_marginTop = UT_String_sprintf ("margin-top: %s;", val);
+    
     val = UT_getAttribute ("fo:margin-right", props);
     if(val)
       m_marginRight = UT_String_sprintf ("margin-right: %s;", val);
   
+    val = UT_getAttribute ("fo:margin-bottom", props);
+    if(val)
+      m_marginBottom = UT_String_sprintf ("margin-bottom: %s;", val);
+    
     val = UT_getAttribute ("style:line-height-at-least", props);
     if (val)
       m_lineHeight = UT_String_sprintf ("line-height: %s+;", val);
@@ -236,7 +430,9 @@ private:
 	m_bPageBreakBefore = true;
     }
 
-#define APPEND_STYLE(sty) if (sty.size()) m_styleProps += sty;
+	m_styleProps = "";
+
+#define APPEND_STYLE(sty) if (sty.size()) { m_styleProps += sty; }
     
     APPEND_STYLE(m_align);
     APPEND_STYLE(m_fontWeight);
@@ -249,7 +445,9 @@ private:
     APPEND_STYLE(m_textPos);
     APPEND_STYLE(m_textDecoration);
     APPEND_STYLE(m_marginLeft);
+    APPEND_STYLE(m_marginTop);
     APPEND_STYLE(m_marginRight);
+    APPEND_STYLE(m_marginBottom);
     APPEND_STYLE(m_lineHeight);
     APPEND_STYLE(m_keepWithNext);
     APPEND_STYLE(m_keepTogether);
@@ -263,6 +461,8 @@ private:
       m_styleProps [m_styleProps.size()-1] = 0; 
   }
 
+private:
+
   OO_Style (); // no impl
 
   UT_String m_align;
@@ -276,7 +476,9 @@ private:
   UT_String m_textPos;
   UT_String m_textDecoration;
   UT_String m_marginLeft;
+  UT_String m_marginTop;
   UT_String m_marginRight;
+  UT_String m_marginBottom;
   UT_String m_lineHeight;
   UT_String m_keepWithNext;
   UT_String m_keepTogether;
@@ -288,6 +490,8 @@ private:
 
   bool m_bColBreakBefore;
   bool m_bPageBreakBefore;
+  const PD_Style * m_pParentStyle;
+  bool m_bOpenDocument;
 }; // class OO_Style
 
 /*****************************************************************************/
@@ -310,16 +514,20 @@ public:
   void defineSimpleStyle (const UT_UTF8String & name, const XML_Char **props);
   const XML_Char* mapStyle (const XML_Char * name) const;
   const OO_Style * mapStyleObj (const XML_Char * name) const;
+  OpenWriter_StylesStream_Listener * m_pSSListener;
 
 private:
 
+  UT_Error _handleMimetype ();
   UT_Error _handleMetaStream ();
   UT_Error _handleSettingsStream ();
   UT_Error _handleStylesStream ();
   UT_Error _handleContentStream ();
 
   GsfInfile * m_oo;
-  UT_StringPtrMap m_styleBucket;
+  UT_GenericStringMap<OO_Style *> m_styleBucket;
+  //! TRUE ... OASIS OpenDocument, FALSE ... OpenOffice.org 1.0 OpenWriter
+  bool m_bOpenDocument;
 };
 
 /*****************************************************************************/
@@ -339,7 +547,8 @@ IE_Imp_OpenWriter_Sniffer::~IE_Imp_OpenWriter_Sniffer ()
  */
 UT_Confidence_t IE_Imp_OpenWriter_Sniffer::recognizeSuffix (const char * szSuffix)
 {
-  if (!UT_stricmp(szSuffix, ".sxw"))
+  if (!UT_stricmp(szSuffix, ".sxw") || 
+	  !UT_stricmp(szSuffix, ".odt"))
     return UT_CONFIDENCE_PERFECT;
   return UT_CONFIDENCE_ZILCH;
 }
@@ -351,8 +560,8 @@ UT_Confidence_t IE_Imp_OpenWriter_Sniffer::recognizeContents (const char * szBuf
 							      UT_uint32 iNumbytes)
 {
   // BOGUS - will identify any zip file, also contains null characters
-  static const char * magic = "PK\x03\x04\x14\0\0\0\0\0";
-  UT_uint32 magic_len = strlen ( magic );
+  static const char magic[10] = {'P','K',0x03,0x04,0x14,'\0','\0','\0','\0','\0'};  
+  UT_uint32 magic_len = sizeof ( magic );
   
   if (iNumbytes < magic_len)
     return UT_CONFIDENCE_ZILCH;
@@ -380,8 +589,8 @@ bool IE_Imp_OpenWriter_Sniffer::getDlgLabels (const char ** szDesc,
 					      const char ** szSuffixList,
 					      IEFileType * ft)
 {
-  *szDesc = "OpenOffice Writer (.sxw)";
-  *szSuffixList = "*.sxw";
+  *szDesc = "OpenOffice Writer (.sxw, .odt)";
+  *szSuffixList = "*.sxw; *.odt";
   *ft = getFileType();
   return true;
 }
@@ -391,13 +600,13 @@ void IE_Imp_OpenWriter::defineSimpleStyle (const UT_UTF8String & name, const XML
   if (!name.size() || !props)
     return;
   
-  OO_Style * style = new OO_Style (props);
+  OO_Style * style = new OO_Style (props, NULL, m_bOpenDocument);
   m_styleBucket.insert (name.utf8_str(), style);
 }
 
 const XML_Char* IE_Imp_OpenWriter::mapStyle (const XML_Char * name) const
 {
-  const OO_Style * style = (const OO_Style *)m_styleBucket.pick((const char *)name);
+  OO_Style * style = m_styleBucket.pick((const char *)name);
   if (NULL == style)
     return "";
   return style->getAbiStyle (); 
@@ -405,7 +614,9 @@ const XML_Char* IE_Imp_OpenWriter::mapStyle (const XML_Char * name) const
 
 const OO_Style * IE_Imp_OpenWriter::mapStyleObj (const XML_Char * name) const
 {
-  return (const OO_Style *)m_styleBucket.pick((const char *)name);
+  if (!name)
+    return NULL;
+  return m_styleBucket.pick((const char *)name);
 }
 
 /*****************************************************************************/
@@ -415,7 +626,7 @@ const OO_Style * IE_Imp_OpenWriter::mapStyleObj (const XML_Char * name) const
  * Create a new OpenWriter importer object
  */
 IE_Imp_OpenWriter::IE_Imp_OpenWriter (PD_Document * pDocument)
-  : IE_Imp (pDocument), m_oo (0)
+  : IE_Imp (pDocument), m_oo (0), m_pSSListener(0), m_bOpenDocument(false)
 {
 }
 
@@ -427,7 +638,7 @@ IE_Imp_OpenWriter::~IE_Imp_OpenWriter ()
   if (m_oo)
     g_object_unref (G_OBJECT(m_oo));
 
-  UT_HASH_PURGEDATA(const OO_Style *, &m_styleBucket, delete);
+  m_styleBucket.purgeData();
 }
 
 /*!
@@ -448,9 +659,9 @@ UT_Error IE_Imp_OpenWriter::importFile(const char * szFilename)
   
   UT_Error err = UT_OK;
 
-  if ( UT_OK != (err = _handleMetaStream ()))
+  if ( UT_OK != (err = _handleMimetype ()))
     return err;
-  if ( UT_OK != (err = _handleStylesStream ()))
+  if ( UT_OK != (err = _handleMetaStream ()))
     return err;
   if ( UT_OK != (err = _handleStylesStream ()))
     return err;
@@ -586,10 +797,16 @@ static UT_Error handleStream ( GsfInfile * oo,
 class ABI_EXPORT OpenWriter_MetaStream_Listener : public OpenWriter_Stream_Listener
 {
 public:
-  OpenWriter_MetaStream_Listener ( IE_Imp_OpenWriter * importer )
-    : OpenWriter_Stream_Listener ( importer )
+  OpenWriter_MetaStream_Listener ( IE_Imp_OpenWriter * importer, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), 
+      m_bOpenDocument ( bOpenDocument )
   {
-    getDocument()->setMetaDataProp(PD_META_KEY_FORMAT, UT_UTF8String ("OpenWriter::SXW"));
+    if (m_bOpenDocument) {  
+      getDocument()->setMetaDataProp(PD_META_KEY_FORMAT, UT_UTF8String ("OpenWriter::ODT"));
+    }
+	else {
+      getDocument()->setMetaDataProp(PD_META_KEY_FORMAT, UT_UTF8String ("OpenWriter::SXW"));
+    }
   }
   
   virtual ~OpenWriter_MetaStream_Listener ()
@@ -634,14 +851,51 @@ private:
 
   UT_UTF8String mCharData;
   UT_UTF8String mAttrib;
+  const bool m_bOpenDocument;
 };
 
+/*!
+ * Determine mimetype (odt or sxw)
+ */
+UT_Error IE_Imp_OpenWriter::_handleMimetype ()
+{
+  guint8 const *data = NULL;
+  size_t len = 0;
+
+  GsfInput * input = gsf_infile_child_by_name(m_oo, "mimetype");
+
+  m_bOpenDocument = FALSE;
+
+  if (!input) {
+    UT_DEBUGMSG(("Error: didn't get a mimetype. Assuming that it's a application/vnd.sun.xml.writer document\n"));
+    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+    return UT_OK;
+  }
+
+  UT_UTF8String mimetype;
+  if (gsf_input_size (input) > 0) {
+    mimetype.append((const char *)gsf_input_read(input, gsf_input_size (input), NULL), gsf_input_size (input));
+  }
+  
+  if (!UT_strcmp("application/vnd.oasis.opendocument.text", mimetype.utf8_str())) {
+    m_bOpenDocument = TRUE;
+  }
+  else if (!UT_strcmp("application/vnd.sun.xml.writer", mimetype.utf8_str())) {
+  }
+  else {
+	UT_DEBUGMSG(("*** Unknown mimetype '%s'\n", mimetype.utf8_str()));  
+  }
+	  
+  g_object_unref (G_OBJECT (input));
+  return UT_OK;
+}
+
 /*!
  * Handle the meta-stream
  */
 UT_Error IE_Imp_OpenWriter::_handleMetaStream ()
 {
-  OpenWriter_MetaStream_Listener listener ( this );
+  OpenWriter_MetaStream_Listener listener ( this, m_bOpenDocument );
   return handleStream (m_oo, "meta.xml", listener);
 }
 
@@ -654,8 +908,9 @@ UT_Error IE_Imp_OpenWriter::_handleMetaStream ()
 class ABI_EXPORT OpenWriter_SettingsStream_Listener : public OpenWriter_Stream_Listener
 {
 public:
-  OpenWriter_SettingsStream_Listener ( IE_Imp_OpenWriter * importer )
-    : OpenWriter_Stream_Listener ( importer )
+  OpenWriter_SettingsStream_Listener ( IE_Imp_OpenWriter * importer, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), 
+      m_bOpenDocument ( bOpenDocument )
   {
   }
 
@@ -674,6 +929,9 @@ public:
   virtual void charData (const XML_Char * buffer, int length)
   {
   }
+  
+private:
+  const bool m_bOpenDocument;
 };
 
 /*!
@@ -681,7 +939,7 @@ public:
  */
 UT_Error IE_Imp_OpenWriter::_handleSettingsStream ()
 {
-  OpenWriter_SettingsStream_Listener listener ( this );
+  OpenWriter_SettingsStream_Listener listener ( this, m_bOpenDocument );
   return handleStream (m_oo, "settings.xml", listener);
 }
 
@@ -694,33 +952,66 @@ UT_Error IE_Imp_OpenWriter::_handleSettingsStream ()
 class ABI_EXPORT OpenWriter_StylesStream_Listener : public OpenWriter_Stream_Listener
 {
 public:
-  OpenWriter_StylesStream_Listener ( IE_Imp_OpenWriter * importer )
-    : OpenWriter_Stream_Listener ( importer ), m_ooStyle (0)
+  OpenWriter_StylesStream_Listener ( IE_Imp_OpenWriter * importer, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), m_ooStyle (0), m_pageMaster(0), m_bOpenDocument(bOpenDocument)
   {
   }
   
   virtual ~OpenWriter_StylesStream_Listener ()
   {
+    m_styleNameMap.purgeData();
+  }
+
+  UT_UTF8String getStyleName(const UT_UTF8String & in) const
+  {
+    UT_UTF8String * name = m_styleNameMap.pick(in.utf8_str());
+    if (!name)
+      return UT_UTF8String(in);
+    else
+      return *name;
   }
   
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
   {
-    if (!strcmp (name, "style:style")) {
+	/* SXW || ODT */
+    if (!UT_strcmp (name, "style:page-master") || !UT_strcmp (name, "style:page-layout")) {
+      m_pageMaster = UT_getAttribute("style:name", atts);
+    }
+    else if (!UT_strcmp (name, "style:master-page")) {
+      const XML_Char * masterName = UT_getAttribute("style:page-master-name", atts);
+      const XML_Char ** pageAtts = m_ooPageStyle.getAbiPageAtts(masterName);
+      getDocument()->setPageSizeFromFile(pageAtts);
+    }
+    else if (!strcmp (name, "style:style")) {
       const XML_Char * attr = NULL;
 
       attr = UT_getAttribute("style:name",atts);
       if (attr)
 	m_name = attr;
 
+      attr = UT_getAttribute("style:display-name",atts);
+      if (attr)
+	m_displayName = attr;
+
       if (m_name != "Standard") {
 	attr = UT_getAttribute("style:parent-style-name",atts);
-	if (attr)
-	  m_parent = attr;
+	if (attr) {
+	  if (!UT_strcmp(attr, "Standard")) {
+	    m_parent = "Normal";
+	  } else {
+	    m_parent = attr;
+	  }
+	}
 	
 	attr = UT_getAttribute("style:next-style-name",atts);
-	if (attr)
-	  m_next = attr;
-	
+	if (attr) {
+	  if (!UT_strcmp(attr, "Standard")) {
+	    m_next = "Normal";
+	  } else {
+	    m_next = attr;
+	  }
+	}
+
 	attr = UT_getAttribute("style:family",atts);
 	if (attr && strcmp (attr, "paragraph") != 0)
 	  m_type = CHARACTER;
@@ -734,22 +1025,50 @@ public:
 	m_type = PARAGRAPH;
       }
       DELETEP(m_ooStyle);
+      m_ooStyle = NULL;
+    }
+    else if (/* SXW || ODT */
+	     (!strcmp (name, "style:properties") || !strcmp (name, "style:page-layout-properties")) && 
+	     m_pageMaster) {
+      // page setup, because we are in page-master subtree
+      m_ooPageStyle.appendPageMaster(m_pageMaster, atts);
+    }
+    else if (!strcmp (name, "style:properties") || 				/* SXW */
+	     !strcmp (name, "style:text-properties") || 		/* ODT */
+	     !strcmp (name, "style:paragraph-properties")) {
+      
+      if (m_ooStyle == NULL) {
+	getDocument()->getStyle (m_parent.utf8_str(), &m_pParentStyle);
+	m_ooStyle = new OO_Style (atts, m_pParentStyle, m_bOpenDocument);
+      }
+      else {
+	m_ooStyle->parse(atts);
+      }
     }
-    else if (!strcmp (name, "style:properties"))
-      m_ooStyle = new OO_Style (atts);
   } 
 
   virtual void endElement (const XML_Char * name)
   {
-    if (!strcmp (name, "style:style")) {
+    if (!UT_strcmp (name, "style:page-master")) {
+      m_pageMaster = NULL;
+    }
+    else if (!strcmp (name, "style:style")) {
       if (m_name.size ()) {
 	int propCtr = 0;
 	
 	const XML_Char * atts[11];
 	atts[propCtr++] = "type";
 	atts[propCtr++] = (m_type == PARAGRAPH ? "P" : "C");
+
 	atts[propCtr++] = "name";
-	atts[propCtr++] = m_name.utf8_str();
+	if (m_displayName.size()) {	 
+	  atts[propCtr++] = m_displayName.utf8_str();
+	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_displayName));
+	}
+	else {
+	  atts[propCtr++] = m_name.utf8_str();
+	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_name));
+	}
 
 	if (m_ooStyle) {
 	  atts[propCtr++] = "props";
@@ -772,24 +1091,42 @@ public:
       }
 
       m_name.clear ();
+      m_displayName.clear ();
       m_parent.clear ();
       m_next.clear ();
+      DELETEP(m_ooStyle);
+      m_ooStyle = NULL;
     }
-
-    DELETEP(m_ooStyle);
   }
   
   virtual void charData (const XML_Char * buffer, int length)
   {
   }  
 
+  const XML_Char * getSectionProps() const 
+  {
+    if (!UT_strcmp (m_ooPageStyle.getName(), "")) 
+    {
+      UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+      return NULL;
+    }
+    return m_ooPageStyle.getAbiSectionProps();
+  }
+  
 private:
 
   UT_UTF8String m_name;
+  UT_UTF8String m_displayName;
   UT_UTF8String m_parent;
   UT_UTF8String m_next;
   enum { CHARACTER, PARAGRAPH } m_type;
   OO_Style *m_ooStyle;
+  PD_Style *m_pParentStyle;
+  OO_PageStyle m_ooPageStyle;
+  const XML_Char * m_pageMaster;
+  const bool m_bOpenDocument;
+
+  UT_GenericStringMap<UT_UTF8String *> m_styleNameMap;
 };
 
 /*!
@@ -797,8 +1134,8 @@ private:
  */
 UT_Error IE_Imp_OpenWriter::_handleStylesStream ()
 {
-  OpenWriter_StylesStream_Listener listener ( this );
-  return handleStream (m_oo, "styles.xml", listener);
+  m_pSSListener = new OpenWriter_StylesStream_Listener(this, m_bOpenDocument);
+  return handleStream (m_oo, "styles.xml", *m_pSSListener);
 }
 
 /*****************************************************************************/
@@ -812,12 +1149,14 @@ class ABI_EXPORT OpenWriter_ContentStream_Listener : public OpenWriter_Stream_Li
 private:
   UT_UCS4String m_charData;
   bool m_bAcceptingText;
-  bool m_bInSection;  
+  bool m_bInSection;
+  bool m_bInTOC;
 
   UT_UTF8String m_curStyleName;
 
-  UT_Vector		m_vecInlineFmt;
-  UT_NumberStack	m_stackFmtStartIndex;
+  UT_GenericVector<XML_Char*>	m_vecInlineFmt;
+  UT_Stack		m_stackFmtStartIndex;
+  const OpenWriter_StylesStream_Listener * m_pSSListener;
 
   UT_uint32 m_imgCnt;
 
@@ -825,10 +1164,12 @@ private:
   int m_col;
   int m_cel;
 
+  const bool m_bOpenDocument;
+
 public:
-  OpenWriter_ContentStream_Listener ( IE_Imp_OpenWriter * importer )
-    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), m_imgCnt(0),
-      m_row(0), m_col(0), m_cel(0)
+  OpenWriter_ContentStream_Listener ( IE_Imp_OpenWriter * importer, OpenWriter_StylesStream_Listener * pSSListener, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), m_bInTOC(false),
+      m_pSSListener(pSSListener), m_imgCnt(0), m_row(0), m_col(0), m_cel(0), m_bOpenDocument(bOpenDocument)
   {
   }
 
@@ -838,6 +1179,10 @@ public:
 
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
   {
+    // if we're inside a TOC, ignore the contents since SXW/ODT include a copy of the TOC
+    if (m_bInTOC)
+      return;
+
     if (!UT_strcmp(name, "office:body"))
       {
       }
@@ -846,22 +1191,12 @@ public:
 	const XML_Char * oo_sty = UT_getAttribute ("text:style-name", atts);
 	const XML_Char * abi_sty = _mapStyle(oo_sty);
 
-	const XML_Char * props[3];
-	props[0] = "props";
-	props[1] = abi_sty;
-	props[2] = 0;
-	getDocument()->appendStrux(PTX_Section, (const XML_Char**)props);	
-	m_bInSection = true;
+	_insureInSection(abi_sty);
       }
     else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
-        if (!m_bInSection)
-	  {
-	      getDocument()->appendStrux(PTX_Section, NULL);
-	      m_bInSection = true;
-	  }
-	const XML_Char * oo_sty = UT_getAttribute ("text:style-name", atts);
-	const OO_Style * abi_sty = _mapStyleObj (oo_sty);
+	UT_UTF8String oo_sty;
+	const OO_Style * abi_sty = _mapStyleObj (UT_getAttribute ("text:style-name", atts), oo_sty);
 
 	if (abi_sty) {
 	  // append an empty paragraph with this one char
@@ -876,18 +1211,18 @@ public:
 	  props[1] = abi_sty->getAbiStyle ();
 	  props[2] = 0;
 	  
-	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
+	  _insureInBlock((const XML_Char **)props);
 	}
-	else if (oo_sty) {
+	else if (oo_sty.size()) {
 	  const XML_Char * props[3];
 	  props[0] = "style";
-	  props[1] = oo_sty;
+	  props[1] = oo_sty.utf8_str();
 	  props[2] = 0;
 
-	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
+	  _insureInBlock((const XML_Char **)props);
 	}
 	else
-	  getDocument()->appendStrux(PTX_Block, NULL);
+	  _insureInBlock(NULL);
 	m_bAcceptingText = true;
       }
     else if (!UT_strcmp(name, "text:span"))
@@ -910,12 +1245,38 @@ public:
 	m_charData += UCS_LF;
 	_flush ();
       }
+    else if (!UT_strcmp(name, "text:ordered-list") || !UT_strcmp(name, "text:unordered-list"))
+      {
+	const XML_Char * list_atts[15];
+	UT_uint32 iOffset = 0;
+
+	// list type
+	list_atts[iOffset++] = "type";
+	if (!UT_strcmp(name, "text:ordered-list"))
+	  list_atts[iOffset++] = "Numbered List";
+	else
+	  list_atts[iOffset++] = "Bullet List";
+
+	list_atts[iOffset++] = "id";
+	list_atts[iOffset++] = "0";
+
+	// NULL
+	list_atts[iOffset++] = 0;  
+
+	getDocument()->appendList(list_atts);
+
+	UT_DEBUGMSG(("DOM: appended a list\n"));
+      }
     else if (!UT_strcmp(name, "style:style"))
       {
 	m_curStyleName.clear ();
 	m_curStyleName = UT_getAttribute ("style:name", atts);
       }
-    else if (!UT_strcmp(name, "style:properties") || !UT_strcmp(name, "style:columns"))
+    else if (/* SXW */
+			!UT_strcmp(name, "style:properties") || 
+			!UT_strcmp(name, "style:columns") ||
+			/* ODT */
+			!UT_strcmp(name, "style:text-properties"))
       {
 	_defineSimpleStyle (atts);
       }
@@ -934,18 +1295,25 @@ public:
       _insertBookmark (name, "start");
       _insertBookmark (name, "end");
     }
+    else if (!UT_strcmp(name, "text:table-of-content"))
+    {
+      _flush ();
+      _insureInBlock(NULL);
+      
+      getDocument()->appendStrux(PTX_SectionTOC, NULL);
+      getDocument()->appendStrux(PTX_EndTOC, NULL);
+
+      m_bInTOC = true;
+    }
     else if (!UT_strcmp(name, "draw:image"))
       {
 	_flush ();
+	_insureInBlock(NULL);
 	_insertImage (atts);
       }
     else if (!UT_strcmp(name, "table:table"))
       {
-        if (!m_bInSection)
-	  {
-	      getDocument()->appendStrux(PTX_Section, NULL);
-	      m_bInSection = true;
-	  }
+	_insureInSection(NULL);
 	_openTable (atts);
       }
     else if (!UT_strcmp(name, "table:table-column"))
@@ -1016,10 +1384,14 @@ public:
 
   virtual void endElement (const XML_Char * name)
   {
-    if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
+    if (!UT_strcmp(name, "text:section" ))
+      {
+	m_bInSection = false;
+      }
+    else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
 	_flush ();
-	m_bAcceptingText= false;
+	m_bAcceptingText = false;
       }
     else if (!UT_strcmp(name, "text:span"))
       {
@@ -1027,11 +1399,17 @@ public:
 	_popInlineFmt();
 	getDocument()->appendFmt(&m_vecInlineFmt);
       }
+    else if (!UT_strcmp(name, "text:ordered-list") || !UT_strcmp(name, "text:unordered-list"))
+      {
+      }
     else if (!UT_strcmp(name, "text:a"))
       {
 	_flush ();
 	getDocument()->appendObject(PTO_Hyperlink, NULL);
       }
+    else if (!UT_strcmp(name, "text:table-of-content")) {
+      m_bInTOC = false;
+    }
     else if (!UT_strcmp(name, "table:table"))
       {
 	_closeTable ();
@@ -1069,7 +1447,7 @@ public:
 
   virtual void charData (const XML_Char * buffer, int length)
   {
-    if (buffer && length && m_bAcceptingText)
+    if (buffer && length && m_bAcceptingText && !m_bInTOC)
       m_charData += UT_UCS4String (buffer, length, true);
   }
 
@@ -1088,14 +1466,21 @@ private:
 
     GsfInfile * pictures_dir = GSF_INFILE(gsf_infile_child_by_name(getImporter()->getOO(), "Pictures"));
 
-    // 10 == strlen("#Pictures/");
-    error = loadStream(pictures_dir, href+10, img_buf);
-    g_object_unref (G_OBJECT (pictures_dir));
+	if (m_bOpenDocument) {
+	    // 9 == strlen("Pictures/");
+	    error = loadStream(pictures_dir, href+9, img_buf);
+	    g_object_unref (G_OBJECT (pictures_dir));
+	}
+	else {
+	    // 10 == strlen("#Pictures/");
+	    error = loadStream(pictures_dir, href+10, img_buf);
+	    g_object_unref (G_OBJECT (pictures_dir));
+	}
 
     if (error != UT_OK)
       return;
 
-    char * mimetype 	= const_cast<char*>(UT_strdup ("image/png"));
+    const char * mimetype 	= UT_strdup ("image/png");
     IE_ImpGraphic * importer	= 0;
     FG_Graphic* pFG		= 0;
     UT_ByteBuf * pictData       = 0;
@@ -1138,10 +1523,10 @@ private:
     UT_String_sprintf(propsName, "image%d", m_imgCnt);
 
     const XML_Char* propsArray[5];
-    propsArray[0] = (const XML_Char *)"props";
-    propsArray[1] = (const XML_Char *)propBuffer.c_str();
-    propsArray[2] = (const XML_Char *)"dataid";
-    propsArray[3] = (const XML_Char *)propsName.c_str();
+    propsArray[0] = (XML_Char *)"props";
+    propsArray[1] = (XML_Char *)propBuffer.c_str();
+    propsArray[2] = (XML_Char *)"dataid";
+    propsArray[3] = (XML_Char *)propsName.c_str();
     propsArray[4] = 0;
     
     if (!getDocument()->appendObject (PTO_Image, propsArray))
@@ -1149,7 +1534,7 @@ private:
 	FREEP(mimetype);
 	goto Cleanup;
       }
-    
+
     if (!getDocument()->createDataItem(propsName.c_str(), false,
 				       pictData, (void*)mimetype, NULL))
       {
@@ -1160,12 +1545,43 @@ private:
     DELETEP(importer);
   }
 
+  void _insureInBlock(const XML_Char ** atts)
+  {
+    if (m_bAcceptingText)
+      return;
+
+    _insureInSection(NULL);
+
+    if (!m_bAcceptingText) {
+      getDocument()->appendStrux(PTX_Block, (const XML_Char**)atts);	
+      m_bAcceptingText = true;
+    }
+  }
+
+  void _insureInSection(const XML_Char * props) 
+  {
+    if (m_bInSection)
+      return;
+
+    UT_String allProps(props);
+    allProps += m_pSSListener->getSectionProps();
+
+    const XML_Char * atts[3];
+    atts[0] = "props";
+    atts[1] = allProps.c_str();
+    atts[2] = 0;
+    getDocument()->appendStrux(PTX_Section, (const XML_Char**)atts);	
+
+    m_bInSection = true;
+    m_bAcceptingText = false;
+  }
+  
   void _insertBookmark (const XML_Char * name, const XML_Char * type)
     {
 	const XML_Char* propsArray[5];
-	propsArray[0] = (const XML_Char *)"name";
+	propsArray[0] = (XML_Char *)"name";
 	propsArray[1] = name;
-	propsArray[2] = (const XML_Char *)"type";
+	propsArray[2] = (XML_Char *)"type";
 	propsArray[3] = type;
 	propsArray[4] = 0;
 	getDocument()->appendObject (PTO_Bookmark, propsArray);
@@ -1214,6 +1630,8 @@ private:
   void _closeTable ()
   {
     getDocument()->appendStrux(PTX_EndTable, NULL);
+
+    m_row = m_cel = m_col = 0;
   }
 
   void _closeColumn ()
@@ -1238,12 +1656,14 @@ private:
 
   const XML_Char* _mapStyle (const XML_Char * name) const
   {
-    return getImporter ()->mapStyle(name);
+    UT_UTF8String styleName = m_pSSListener->getStyleName(name);
+    return getImporter ()->mapStyle(styleName.utf8_str());
   }
 
-  const OO_Style * _mapStyleObj (const XML_Char * name) const
+  const OO_Style * _mapStyleObj (const XML_Char * name, UT_UTF8String & oo_sty) const
   {
-    return getImporter ()->mapStyleObj (name);
+    oo_sty = m_pSSListener->getStyleName(name);
+    return getImporter ()->mapStyleObj (oo_sty.utf8_str());
   }
 
   bool _pushInlineFmt(const XML_Char ** atts)
@@ -1259,24 +1679,24 @@ private:
 	if (m_vecInlineFmt.addItem(p)!=0)
 	  return false;
       }
-    if (!m_stackFmtStartIndex.push(start))
+    if (!m_stackFmtStartIndex.push((void*)start))
       return false;
     return true;
   }
   
   void _popInlineFmt(void)
   {
-    UT_sint32 start;
-    if (!m_stackFmtStartIndex.pop(&start))
+    UT_uint32 start;
+    if (!m_stackFmtStartIndex.pop((void **)&start))
       return;
     UT_uint32 k;
     UT_uint32 end = m_vecInlineFmt.getItemCount();
-    for (k=end; k>=static_cast<UT_uint32>(start); k--)
+    for (k=end; k>=start; k--)
       {
-	XML_Char * p = const_cast<XML_Char*>((const XML_Char *)m_vecInlineFmt.getNthItem(k-1));
+	const XML_Char * p = (const XML_Char *)m_vecInlineFmt.getNthItem(k-1);
 	m_vecInlineFmt.deleteNthItem(k-1);
 	if (p)
-	  FREEP(p);
+	  free((void *)p);
       }
   }
   
@@ -1287,6 +1707,6 @@ private:
  */
 UT_Error IE_Imp_OpenWriter::_handleContentStream ()
 {
-  OpenWriter_ContentStream_Listener listener ( this );
+  OpenWriter_ContentStream_Listener listener (this, m_pSSListener, m_bOpenDocument);
   return handleStream (m_oo, "content.xml", listener);
 }
